//package teikade.debug;

import sun.tools.debug.RemoteStackFrame;
import java.util.*;
import java.awt.*;

public class DebuggerView extends Panel implements Observer {
    Debugger model;
    List stackListView;
    InspectorView objInspectorView;
    InspectorView stackInspectorView;
    TextArea codeView;

    public static void openOn(Debugger debugger) {
	DebuggerView view = new DebuggerView(debugger);
	ApplicationFrame frame = new ApplicationFrame();
	frame.setLayout(new BorderLayout());
	frame.add("Center", view);
	frame.pack();
	frame.resize(400, 400);
	frame.show();
    }

    DebuggerView(Debugger model) {
	this.model = model;
	model.addObserver(this);

	// create views
	stackListView = new List();
	objInspectorView = new InspectorView(model.objInspector);
	stackInspectorView = new InspectorView(model.stackInspector);
	codeView = new TextArea();

	Panel inspectors = new Panel();
	inspectors.setLayout(new GridLayout(0, 2));
	inspectors.add(objInspectorView);
	inspectors.add(stackInspectorView);

/*
	Panel p = new Panel();
	p.setLayout(new BorderLayout());
	p.add("North", inspectors);
	p.add("Center", codeView);
*/

	setLayout(new GridLayout(3, 0));
	add(stackListView);
	add(inspectors);
	add(codeView);
    }

    public boolean handleEvent(Event evt) {
	if (evt.id == Event.WINDOW_DESTROY) {
	    getParent().hide();
	    //getParent().dispose();
	}
	if (evt.target == stackListView) {
	    model.selected(stackListView.getSelectedIndex());
	}
	return super.handleEvent(evt);
    }

    /**
     * java.util.Observer stuff
     */
    public void update(Observable o, Object arg) {
	if (arg == Debugger.LIST) {
	    stackListView.clear();
	    RemoteStackFrame list[] = model.list;
	    for (int i = 0; i < list.length; i++) try {
		stackListView.addItem(list[i].getRemoteClass().getName() +
				"#" + list[i].getMethodName() +
				":" + list[i].getLineNumber());
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	    update(o, Debugger.SELECTION);
	} else if (arg == Debugger.SELECTION) {
	    codeView.setText(model.getSourceCode(stackListView.getRows()));
	}
	invalidate();
    }
}
