/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui;

import teikade.sys.CodePath;
import teikade.sys.ProjectManager;
import teikade.sys.PackageManager;
import teikade.sys.SourceFileManager;
import teikade.sys.ClassManager;
import teikade.sys.MethodManager;
import teikade.sys.Manager;
import teikade.sys.TeikadeProperties;
import teikade.sys.TeikadeException;
import teikade.util.CodeLocation;
import teikade.util.CodeLocationException;
import teikade.util.StringSorter;
import teikade.lang.AccessFlags;
import teikade.lang.ClassSpec;
import teikade.debug.DebuggerManager;
import teikade.gui.component.DList;
import teikade.gui.component.SplitPanel;
import teikade.gui.component.SplitHandle;
import teikade.gui.component.SplitLayout;
import teikade.gui.menu.TeikadeMenuBar;
import teikade.gui.menu.BrowserMenuBar;
import teikade.gui.menu.DebugMenu;
import teikade.gui.dialog.NewPackageDialog;
import teikade.gui.dialog.NewClassDialog;
import teikade.gui.dialog.NewMethodDialog;
import java.util.Enumeration;
import java.awt.Component;
import java.awt.Panel;
import java.awt.List;
import java.awt.Event;

public class Browser extends TeikadeFrame {
    protected DList pkgList;
    protected DList clsList;
    protected DList mtdList;

    protected DebugMenu debugMenu;
    
    public static final int NO_SORT = 0;
    public static final int SORT_BY_METHOD_NAME = 1;
    public static final int SORT_BY_RETURN_TYPE = 2;
    private int methodOrder = NO_SORT;

    private static final String SORT_SEPARATOR = " : ";
    private static final int SORT_SEPARATOR_LENGTH = SORT_SEPARATOR.length();

    private static final String PROPERTY_LIST_LINES =
	"attribute.browsing.lines";
    private static final String PROPERTY_LIST_HEIGHT =
	"attribute.browsing.list.height.";
    private static final String PROPERTY_LIST_WIDTH_PACKAGE =
	"attribute.browsing.list.width.package.";
    private static final String PROPERTY_LIST_WIDTH_CLASS =
	"attribute.browsing.list.width.class.";
    private static final String PROPERTY_ORDER_METHOD =
	"attribute.browsing.order.method.";

    AccessFlags getHideMethods() {
        return ((BrowserMenuBar)getMenuBar()).hideMethods();
    }

    /**
     * spawn browser
     */
    public static Browser openOn(CodePath cp) {
	return openOn(cp, cp);
    }

    /**
     * spawn browser
     */
    static Browser openOn(CodePath cp, CodePath select) {
	StringBuffer title = new StringBuffer(cp.printString());
	title.append(" - ");
	if (cp.methodManager() != null) {
	    title.append("Method");
	} else if (cp.classManager() != null) {
	    title.append("Class");
	} else if (cp.packageManager() != null) {
	    title.append("Package");
	} else {
	    title.append("Project");
	}
	title.append(" Browser");
	return openOn(cp, select, title.toString());
    }

    /**
     * spawn browser
     */
    static Browser openOn(CodePath cp, CodePath select, String title) {
	Browser browser = new Browser(cp, select, title);
	browser.open();
	return browser;
    }

    /**
     * the constructor.
     */
    public Browser(CodePath cp, CodePath select, String title) {
	super(cp, title);
	current = new CodePath(select);
	int order = TeikadeProperties.getInteger(PROPERTY_ORDER_METHOD
						 + startup.projectName());
	if (order > 0 && order < 3) methodOrder = order;
    }

    /**
     * create menubar items
     */
    protected TeikadeMenuBar createMenuBar() {
	//debugMenu = new DebugMenu("Debug", this);
	BrowserMenuBar menubar = new BrowserMenuBar(debugMenu);
	int order = TeikadeProperties.getInteger(PROPERTY_ORDER_METHOD
						 + startup.projectName());
	if (order > 0 && order < 3) methodOrder = order;
	menubar.updateMethodOrder(methodOrder);
	return menubar;
    }

    /**
     * create display items
     */
    protected Component createDisplayItems() {
	Panel panel = new Panel();
	panel.setLayout(new SplitLayout(SplitLayout.VERTICAL));
	int listLines = TeikadeProperties.getInteger(PROPERTY_LIST_LINES);
	if (startup.methodManager() == null) {
	    SplitPanel listArea = new SplitPanel(SplitPanel.HORIZONTAL);
	    if (startup.packageManager() == null) {	// package selection list is required
        	pkgList = new DList(listLines, false);
		listArea.add(propertyNameOfPackageListWidth(startup.projectName()),
			     pkgList.makePanel());
	    }
	    if (startup.classManager() == null) {	// class selection list is required
		if (listArea.countComponents() > 0) {
		    listArea.addSplitHandle();
		}
        	clsList = new DList(listLines, false);
		listArea.add(propertyNameOfClassListWidth(startup.projectName()),
			     clsList.makePanel());
	    }
	    if (listArea.countComponents() > 0) {
		listArea.addSplitHandle();
	    }
	    mtdList = new DList(listLines, false);
	    listArea.add(mtdList.makePanel());

	    panel.add(PROPERTY_LIST_HEIGHT + startup.projectName(), listArea);
	    panel.add(new SplitHandle());
	}
	panel.add(super.createDisplayItems());
	codePathField.setObserver(this);
	return panel;
    }

    protected String propertyNameOfPackageListWidth(String projectName) {
	return PROPERTY_LIST_WIDTH_PACKAGE + projectName;
    }

    protected String propertyNameOfClassListWidth(String projectName) {
	return PROPERTY_LIST_WIDTH_CLASS + projectName;
    }

    private String methodPrototypeForList(String methodPrototype) {
	if (methodPrototype != null && methodOrder == SORT_BY_METHOD_NAME) {
	    int space = methodPrototype.indexOf(' ');
	    if (space >= 0) {
		methodPrototype = methodPrototype.substring(space + 1)
				+ SORT_SEPARATOR
				+ methodPrototype.substring(0, space);
	    }
	}
	return methodPrototype;
    }

    protected String getSelectedMethodPrototype() {
	String selectedItem = mtdList.getSelectedItem();
	if (selectedItem != null && methodOrder == SORT_BY_METHOD_NAME) {
	    int separator = selectedItem.indexOf(SORT_SEPARATOR);
	    if (separator > 0) {
		selectedItem =
		    selectedItem.substring(separator + SORT_SEPARATOR_LENGTH)
		  + " "
		  + selectedItem.substring(0, separator);
	    }
	}
	return selectedItem;
    }

    public boolean handleEvent(Event ev) {
	switch (ev.id) {
	case Event.LIST_SELECT:
	case Event.LIST_DESELECT:
	case Event.ACTION_EVENT:
	    CodePath selected = new CodePath(current);
	    if (ev.target == pkgList) {
		selected.packageManager(pkgList.getSelectedItem());
	    } else if (ev.target == clsList) {
		selected.classManager(clsList.getSelectedItem());
	    } else if (ev.target == mtdList) {
		selected.methodManager(getSelectedMethodPrototype());
	    } else {
		break;
	    }
	    if (!codeData.isChanged()) {
		current = selected;
		try {
		    if (ev.target == pkgList) {
			updateClasses();
		    } else if (ev.target == clsList) {
			updateMethods();
		    } else if (ev.target == mtdList) {
			updateCurrent();
		    }
		}
		catch (TeikadeException e) {
		    current.classManager((ClassManager)null);
		    update();
		}
	    } else if (acceptWithConfirm()) {
		current = selected;
		update();
	    } else {
		if (ev.target == pkgList) {
		    selectItem(pkgList, current.packageName());
		} else if (ev.target == clsList) {
		    selectItem(clsList, current.className());
		} else if (ev.target == mtdList) {
		    selectItem(mtdList,
			    methodPrototypeForList(current.methodPrototype()));
		}
	    }
	    SourceFileManager sfm = current.sourceFileManager();
	    if (sfm != null && sfm.dirty) {
		sfm.dirty = false;
		updateNotify(new CodePath(sfm));
	    }
	    return true;
	}
	SourceFileManager sfm = current.sourceFileManager();
	if (sfm != null && sfm.dirty) {
	    sfm.dirty = false;
	    updateNotify(new CodePath(sfm));
	}
	return super.handleEvent(ev);
    }

    public boolean postEvent0(Event ev) {
	if ((ev.target instanceof List)
	&&  (ev.id == Event.KEY_PRESS || ev.id == Event.KEY_ACTION)) {
	    return handleEvent(ev);
	}
	return super.postEvent0(ev);
    }

    public boolean select(CodePath cp) {
	if (!cp.matches(current)) {
	    if (!acceptWithConfirm()) return false;
	    current = cp;
	}
	update();
	return true;
    }

    private void selectItem(List list, String name) {
	int num = list.countItems();
	for (int i = 0; i < num; i++) {
	    String listedName = list.getItem(i);
	    if (listedName.equals(name)) {
		if (!list.isSelected(i)) list.select(i);
		return;
	    }
	}
    }

    /**
     * menu handler of new package
     */
    public void newPackage(CodePath cp) {
	if (!acceptWithConfirm()) return;
	NewPackageDialog dialog = new NewPackageDialog(this);
	if (dialog.confirm() && acceptWithConfirm()) {
	    PackageManager pm = cp.projectManager().createPackageAfter(
				dialog.getName(), cp.packageManager());
	    if (pm != null) {
		current.packageManager(pm);
		TeikadeFrame.updateNotify(current);
		update();
	    }
	}
	dialog.dispose();
    }

    /**
     * new class or interface
     */
    public void newClass(CodePath cp, boolean isInterface) {
	if (!acceptWithConfirm()) return;
	NewClassDialog dialog = (new NewClassDialog(this, current,
				 new ClassSpec(null, null, null, isInterface)));
	if (dialog.confirm() && acceptWithConfirm()) {
	    ClassSpec cs = dialog.getClassSpec();
	    ClassManager cm = cp.packageManager().createClass(cs);
	    if (cm != null) {
		doReset();
		current.classManager(cm);
		TeikadeFrame.updateNotify(current);
		update();
		cm.sync();
	    }
	}
	dialog.dispose();
    }

    /**
     * new constracor or method
     */
    public void newMethod(CodePath cp, boolean isConstractor) {
	if (!acceptWithConfirm()) return;
	NewMethodDialog dialog = null;
	try {
	    dialog = new NewMethodDialog(this, current,
				     cp.classManager().spec(), isConstractor);
	    if (dialog.confirm() && acceptWithConfirm()) {
		codeData.replaceCode("    " +
				 dialog.getDescription() +
				 (dialog.getMethodSpec().isAbstract() ||
				  dialog.getMethodSpec().isNative() ?
					 ";\n" : " {\n    }\n"));
	    }
	}
	catch (TeikadeException e) {
	    information(e.getMessage());
	} finally {
	    if (dialog != null) dialog.dispose();
	}
    }

    /**
     * new constracor or method
     */
    public void newMethod(CodePath cp, String code) {
	if (acceptWithConfirm()) {
	    codeData.replaceCode(code);
	}
    }

    /**
     * remove the structure
     */
    public void remove(CodePath cp) {
	if (confirm("remove " + cp.printShortString() + ", really?")) {
	    doReset();
	    if (cp.getManager().remove() == null) {
		cp.setManager(null);
		TeikadeFrame.updateNotify(cp);
		update();
	    }
	}
    }

    /**
     * rename the structure
     */
    public void rename(CodePath cp) {
	Manager man = cp.getManager();
	String oldName = man.name();
	int idx = oldName.indexOf('(');
	if (idx >= 0) oldName = oldName.substring(0, idx);
	String name = request("rename " + cp.printString() + " to what?", oldName);
	if (name != null && !name.equals(oldName)) {
	    try {
		if (man.rename(name) != null) {
		    TeikadeFrame.updateNotify(cp);
		    update();
		}
	    }
	    catch (TeikadeException e) {
	    }
	}
    }

    /**
     * update all of this frame
     */
    protected void update() {
	try {
	    if (pkgList != null) {
		updatePackages();
	    } else if (clsList != null) {
		updateClasses();
	    } else if (mtdList != null) {
		updateMethods();
	    } else {
		updateCurrent();
	    }
	}
	catch (TeikadeException e) {
	    current.classManager((ClassManager)null);
	    update();
	}
    }

    /**
     * project selection has been changed
     * this method is not called by interactive select operation
     */
    protected void updatePackages() throws TeikadeException {
	if (pkgList == null) {
	    // no package list area
	} else if (current.projectManager() == null) {
	    // no selected project name, clear package list
	    pkgList.clear();
	} else {
	    // check list same as last displaied list
	    ProjectManager pm = current.projectManager();
	    String pkgName = current.packageName();
	    if (!pkgList.selectItemIfSameList(pm.packageNames(), pkgName)) {
		if (pkgList.listAndSelectItem(pm.packageNames(), pkgName) < 0) {
		    current.packageManager((PackageManager)null);
		}
	    }
	}
	updateClasses();
    }

    /**
     * package selection has been changed
     */
    public void updateClasses() throws TeikadeException {
	if (clsList == null) {
	    // no class list area
	} else if (current.packageManager() == null) {
	    // no selected package name, clear class list
	    clsList.clear();
	} else {
	    // check list same as last displaied list
	    PackageManager pm = current.packageManager();
	    String className = current.className();
	    if (!clsList.selectItemIfSameList(pm.classNames(), className)) {
		if (clsList.listAndSelectItem(pm.classNames(), className) < 0) {
		    current.classManager((ClassManager)null);
		}
	    }
	}
	updateMethods();
    }

    /**
     * class selection has been changed
     */
    public void updateMethods() throws TeikadeException {
	if (mtdList == null) {
	    // no method list area
	} else if (current.classManager() == null) {
	    // no selected class name, clear method list
	    mtdList.clear();
	} else {
	    // check list same as last displaied list
	    ClassManager cm = current.classManager();
	    AccessFlags hideMethods = getHideMethods();
	    String methodPrototype = methodPrototypeForList(
						    current.methodPrototype());
	    if (methodOrder == NO_SORT) {
		if (!mtdList.selectItemIfSameList(
			cm.methodPrototypes(hideMethods), methodPrototype)
		&&  mtdList.listAndSelectItem(cm.methodPrototypes(hideMethods),
					      methodPrototype) < 0) {
		    current.methodManager((MethodManager)null);
		}
	    } else {
		StringSorter sorter = new StringSorter();
		Enumeration methods = cm.methodPrototypes(hideMethods);
		while (methods.hasMoreElements()) {
		    sorter.addElementAndSort(methodPrototypeForList(
					     methods.nextElement().toString()));
		}
		if (!mtdList.selectItemIfSameList(sorter.elements(), methodPrototype)
		&&  mtdList.listAndSelectItem(sorter.elements(), methodPrototype) < 0) {
		    current.methodManager((MethodManager)null);
		}
	    }
	}
	updateCurrent();
    }

    protected void updateMenuActivity() {
	if (debugMenu != null) debugMenu.updateActivity(current);
	super.updateMenuActivity();
    }

    public static void updateDebugger() {
	Enumeration en = allFrames();
	while (en.hasMoreElements()) {
	    Object frame = en.nextElement();
	    if (frame instanceof Browser) {
		DebugMenu debugMenu = ((Browser)frame).debugMenu;
		if (debugMenu != null) debugMenu.updateDebugger();
	    }
	}
    }

    public void setMethodOrder(int order) {
	methodOrder = order;
	try {
	    updateMethods();
	}
	catch (TeikadeException e) {
	}
	TeikadeProperties.put(PROPERTY_ORDER_METHOD + startup.projectName(),
			      "" + order);
    }

    public void viewSource(String className, int lineNumber) {
	PackageManager pm = current.packageManager();
	int dot = className.lastIndexOf('.');
	String pkgName = (dot > 0) ? className.substring(0, dot) : "java.lang";
	pm = PackageManager.findPackage(pkgName);
	ClassManager cm = pm.findClass(className.substring(dot + 1));
	if (cm == null) {
	    information("class not found: " + className);
	} else {
	    String fileName = cm.getCodePath().sourceFileManager().name();
	    CodeLocation location;
	    try {
		location = new CodeLocation(current, fileName, lineNumber, null, 0, -1);
	    } catch (CodeLocationException e) {
	        information("not found line " + lineNumber + " in " + className);
		return;
	    }
	    current = location.codePath();
	    update();
	    int cursor[] = location.cursorPos();
	    codeData.select(cursor[0], cursor[1]);
	}
    }

    public void setBreakPoint(DebuggerManager debugger) {
	int lineNumber = lineNumberInFile();
	if (lineNumber >= 0) {
	    debugger.setBreakPoint(current.classManager().fullClassName(), lineNumber);
	}
    }

    public void resetBreakPoint(DebuggerManager debugger) {
	int lineNumber = lineNumberInFile();
	if (lineNumber >= 0) {
	    debugger.resetBreakPoint(current.classManager().fullClassName(), lineNumber);
	}
    }

    private int lineNumberInFile() {
	ClassManager classManager = current.classManager();
	if (classManager == null) return -1;
	MethodManager methodManager = current.methodManager();
	int lineNumber;
	if (methodManager != null) {
	    lineNumber = methodManager.startLine();
	} else {
	    lineNumber = classManager.startLine();
	}
	String code = codeData.getCode();
	int cursor = codeData.getSelectionStart();
	String lineSeparator = System.getProperty("line.separator");
	int c = -1;
	while ((c = code.indexOf(lineSeparator, c + 1)) >= 0 && c < cursor) {
	    lineNumber++;
	}
	return lineNumber;
    }

    public DebuggerManager currentDebugger() {
	return debugMenu == null ? null : debugMenu.currentDebugger();
    }
}
