/*
 * Copyright (c) 1995 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui;

import java.awt.*;
import teikade.gui.component.*;
import teikade.util.ThreadUtil;

public class MemoryMonitor extends DFrame implements Runnable {
    private Button updateButton = new DButton("update now");
    private Choice updateChoice = new Choice();
    private int updateTime = 5 * 1000;
    private Label usedLabel = new DLabel("000,000,000", Label.RIGHT);
    private Label totalLabel = new DLabel("000,000,000", Label.RIGHT);
    private Label percentLabel = new DLabel("000.0", Label.LEFT);
    private Button garbageCorrect = new DButton("garbage collect");
    private Slider slider = new Slider(100, 32);
    private Button closeButton = new DButton("close");
    private Thread updateThread;
    private static MemoryMonitor memoryMonitor = null;

    public static void showOn() {
	if (memoryMonitor == null) {
	    memoryMonitor = new MemoryMonitor();
	}
	memoryMonitor.show();
    }

    private MemoryMonitor() {
	super("memory monitor");
	setBackground(Color.lightGray);
	setFont(new Font("Dialog", Font.PLAIN, 10));

        updateChoice.addItem("1");
        updateChoice.addItem("2");
        updateChoice.addItem("3");
        updateChoice.addItem("5");
        updateChoice.addItem("10");
        updateChoice.addItem("20");
        updateChoice.addItem("30");
        updateChoice.addItem("60");
        updateChoice.addItem("stop");
	updateChoice.select(Integer.toString(updateTime/1000));

	Panel north = new Panel();
	north.add(new DLabel("update every"));
	north.add(updateChoice);
	north.add(new DLabel("sec. "));
	north.add(updateButton);

	Panel labels = new Panel();
	labels.setLayout(new GridLayout(2, 1));
	labels.add(new DLabel( "used memory", Label.RIGHT));
	labels.add(new DLabel("total memory", Label.RIGHT));

	Panel memories = new Panel();
	memories.setLayout(new GridLayout(2, 1));
	memories.add(usedLabel);
	memories.add(totalLabel);

	Panel bytes = new Panel();
	bytes.setLayout(new GridLayout(2, 1));
	bytes.add(new DLabel("bytes"));
	bytes.add(new DLabel("bytes"));

	Panel percents = new Panel();
	percents.setLayout(new GridLayout(2, 1));
	percents.add(percentLabel);
	percents.add(garbageCorrect);

	Panel north2 = new Panel();
	north2.add(labels);
	north2.add(memories);
	north2.add(bytes);
	north2.add(percents);

	Panel center = new Panel();
	center.setLayout(new BorderLayout());
	center.add("North", north2);
	center.add("Center", slider);

	Panel south = new Panel();
	south.add(closeButton);

	add("North", north);
	add("Center", center);
	add("South", south);
	pack();
    }

    public void show() {
	super.show();
	start();
    }

    void start() {
	stop();
	updateThread = new Thread(this,this.getClass().getName());
	updateThread.start();
    }

    void stop() {
	if (updateThread != null) {
	    //updateThread.stop();
	    ThreadUtil.Stop(updateThread);
	    updateThread = null;
	}
    }

    public void run() {
	try {
	    while (updateThread != null) {
		repaint();
		Thread.sleep(updateTime);
	    }
	} catch (InterruptedException e) {
	    updateThread = null;
	}
    }

    public void update(Graphics g) {
	paint(g);
    }

    public void paint(Graphics g) {
	Runtime runtime = Runtime.getRuntime();
	long total = runtime.totalMemory();
	long used  = total - runtime.freeMemory();
	int percent = (int)(used * 1000 / total);

	usedLabel.setText(sizeToString(used));
	totalLabel.setText(sizeToString(total));
	percentLabel.setText("(" + (percent / 10) + "." + (percent % 10) + "%)");
	slider.setValue(percent);
	slider.repaint();
    }

    String sizeToString(long size) {
	StringBuffer stringBuffer = new StringBuffer();
	stringBuffer.append(size);
	int i = stringBuffer.length();
	while ((i -= 3) > 0) {
	    stringBuffer.insert(i, ',');
	}
	return stringBuffer.toString();
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target == updateButton) {
	    repaint();
	} else if (ev.target == updateChoice) {
	    stop();
	    String choice = (String)arg;
	    if (Character.isDigit(choice.charAt(0))) {
		updateTime = Integer.parseInt(choice) * 1000;
		start();
	    }
	} else if (ev.target == garbageCorrect) {
	    Runtime.getRuntime().gc();
	    repaint();
	} else if (ev.target == closeButton) {
	    close();
	} else {
	    return false;
	}
	return true;
    }

    public boolean close() {
	memoryMonitor = null;
	stop();
	super.close();
	return true;
    }

    public static void main(String args[]) {
	new MemoryMonitor().show();
    }
}

class Slider extends Canvas {
    int val;	// 0 - 1000 (1000 is 100%)

    Slider(int width, int height) {
	super();
	setForeground(Color.green);
	resize(width, height);
    }

    public void setValue(int val) {
	if (val < 0)   val = 0;
	if (val > 1000) val = 1000;
	this.val = val;
    }

    public void paint(Graphics g) {
	super.paint(g);

	Dimension d = size();
	int w = d.width - 5 * 2 - 2;
	int h = d.height - 5 * 2 - 2;
	g.setColor(getParent().getBackground());
	g.draw3DRect(5, 5, w, h, false);
	g.draw3DRect(6, 6, w-2, h-2, false);
	w -= 4;
	h -= 4;
	w = w * val / 1000;
	g.setColor(getForeground());
	g.draw3DRect(7, 7, w, h, true);
	g.draw3DRect(8, 8, w-2, h-2, true);
	g.fillRect(9, 9, w-3, h-3);
	getToolkit().sync();
    }
}
