/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;
import teikade.gui.Transcript;

public class DDialog extends Panel implements UseProperties {
    private Dialog peer;

    public DDialog(Frame frame, boolean modal) {
	setLayout(new BorderLayout());
	if (SystemManager.onJDK1_1) {
	    peer = new DDialog1_1(this, frame, modal);
	} else {
	    peer = new DDialog1_0(this, frame, modal);
	}
	peer.setTitle("dialog");
	peer.add("Center", this);
    }

    public void updateProperties() {
	Font font = TeikadeProperties.getFont("font.dialog");
	if (font != null && !equals(peer.getFont())) {
	    setFont(font);
	}
    }

    public void setTitle(String title) {
	peer.setTitle(title);
    }

    public void pack() {
	peer.pack();
    }

    public Frame getFrame() {
	return getFrame(this);
    }

    public static Frame getFrame(Component comp) {
	for (Container c = comp.getParent(); c != null; c = c.getParent()) {
	    if (c instanceof Frame) return (Frame)c;
	}
	return Transcript.getDefaultFrame();
    }

    public void show() {
	Frame frame = getFrame();
	frame.show();
	frame.toFront();
	frame.requestFocus();
	peer.pack();
	if (SystemManager.onWin32) {
	    Dimension size = peer.size();
	    if (size.height == size().height) {
		Frame adjust = new Frame("adjust");
		adjust.pack();
		adjust.show();
		Insets insets = peer.insets();
		peer.resize(size.width, size.height + adjust.size().height);
		adjust.dispose();
	    }
	}
	Rectangle bounds = frame.bounds();
	Dimension me = peer.size();
	Dimension screen = peer.getToolkit().getScreenSize();
	if (bounds.x < 0 || bounds.x + bounds.width  >= screen.width
	||  bounds.y < 0 || bounds.y + bounds.height >= screen.height) {
	    bounds.x = 0;
	    bounds.y = 0;
	    bounds.width  = screen.width;
	    bounds.height = screen.height;
	}
	peer.move(bounds.x + (bounds.width  - me.width ) / 2,
		  bounds.y + (bounds.height - me.height) / 2);
	peer.show();
    }

    public void hide() {
	peer.hide();
    }

    public void dispose() {
	peer.dispose();
    }
}
