/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;
import teikade.gui.editor.EditArea;
import teikade.gui.editor.EditField;

public class DFrame extends DelegateEventFrame implements UseProperties {

    public DFrame() {
	super();
    }

    public DFrame(String title) {
	super(title);
    }

    public void updateProperties() {
	Font font = TeikadeProperties.getFont("font.default");
	if (font != null && !font.equals(getFont())) {
	    setFont(font);
	}
    }

    public boolean postEvent0(Event ev) {
	switch (ev.id) {
	case Event.LIST_SELECT:
	case Event.LIST_DESELECT:
	case Event.ACTION_EVENT:
	    int saveCursor = getCursorType();
	    setCursor(Frame.WAIT_CURSOR);
	    getToolkit().sync();
	    Thread.yield();
	    try {
		return super.postEvent0(ev);
	    } finally {
		setCursor(saveCursor);
	    }
	case Event.WINDOW_DESTROY:
	    close();
	    return true;
	case Event.KEY_ACTION_RELEASE:
	    if (ev.key == Event.F4 && ev.modifiers == Event.ALT_MASK) {
		close();
		return true;
	    }
	    break;
	}
	return super.postEvent0(ev);
    }

    public final boolean mouseEnter(Event ev, int x, int y) {
	if (teikade.sys.SystemManager.onSolaris
	&& !teikade.sys.SystemManager.onJDK1_2) {
	    Component comp = locate(x, y);
	    if (comp != null) {
		while (comp instanceof Container) {
		    Point location = comp.location();
		    x -= location.x;
		    y -= location.y;
		    Component inside = comp.locate(x, y);
		    if (inside == null || inside == comp) break;
		    comp = inside;
		}
		comp.requestFocus();
		return true;
	    }
	}
	return super.mouseEnter(ev, x, y);
    }

    public boolean close() {
	hide();
	dispose();
	return true;
    }
}
