/*
 * Copyright (c) 1995 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import java.util.Enumeration;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;

public class DList extends List implements UseProperties {
    int lastSelectedIndex = -1;
    Panel panel = null;

    public DList() {
	super();
	updateProperties();
    }

    public DList(int rows, boolean multipleSelections) {
	super(rows, multipleSelections);
	updateProperties();
    }

    public void addItem(String item, int index) {
	super.addItem(item, index);
	lastSelectedIndex = getSelectedIndex();
    }

    public void replaceItem(String item, int index) {
	super.replaceItem(item, index);
	lastSelectedIndex = getSelectedIndex();
    }

    public void clear() {
	lastSelectedIndex = -1;
	if (countItems() > 0) {
	    if (SystemManager.onWin32) {
		delItems(0, countItems() - 1);
	    } else {
		super.clear();
	    }
	    if (panel != null && SystemManager.onSolaris) {
		addItem(""); // to clear horizontal scroll bar
		super.clear();
		reshape();
	    }
	}
    }

    public void delItems(int start, int end) {
	super.delItems(start, end);
	lastSelectedIndex = getSelectedIndex();
    }

    public void select(int i) {
	super.select(i);
	lastSelectedIndex = getSelectedIndex();
    }

    public void deselect(int i) {
	super.deselect(i);
	lastSelectedIndex = getSelectedIndex();
    }

    public boolean postEvent(Event event) {
	if ((SystemManager.onJDK1_1 || SystemManager.onWin32)
	&&  event.id == Event.LIST_SELECT) {
	    int selectedIndex = getSelectedIndex();
	    if (selectedIndex == lastSelectedIndex) {
		int deselectIndex = lastSelectedIndex;
		lastSelectedIndex = -1;
		event.id = Event.LIST_DESELECT;
		super.deselect(deselectIndex);
	    } else {
		lastSelectedIndex = selectedIndex;
	    }
	}
	return super.postEvent(event);
    }

    public void updateProperties() {
	Font font = TeikadeProperties.getFont("font.list");
	if (font != null && !font.equals(getFont())) {
	    setFont(font);
	}
	Color color = TeikadeProperties.getColor("color.list.foreground");
	if (color != null) setForeground(color);
	color = TeikadeProperties.getColor("color.list.background");
	if (color != null) setBackground(color);
    }

    public Panel makePanel() {
	panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add("Center", this);
	return panel;
    }

    public void reshape() {
	if (panel != null && SystemManager.onSolaris) {
	    Dimension size = panel.size();
	    resize(size.width, size.height + 1);
	    resize(size);
	}
    }

    public void selectAndVisible(int select) {
	int selected = getSelectedIndex();
	if (select < 0) {
	    if (selected >= 0) deselect(selected);
	} else {
	    if (selected != select) select(select);
	    int rows = getRows() - 1;
	    int half = rows / 2;
	    int visibleIndex = getVisibleIndex();
	    if (SystemManager.onWin32) {
		if (visibleIndex < 0) visibleIndex = 0;
		if (select <= visibleIndex || select >= visibleIndex + rows) {
		    makeVisible(Math.max(select - half, 0));
		}	
	    } else {
		if (visibleIndex < rows) visibleIndex = rows;
		if (select <= visibleIndex - rows || select >= visibleIndex) {
		    makeVisible(Math.min(select + half, countItems() - 1));
		}
	    }
	}
    }

    public boolean selectItemIfSameList(Enumeration en, String name) {
	int num = countItems();
	for (int i = 0; i < num; i++) {
	    if (!en.hasMoreElements()) return false;
	    String listedName = getItem(i);
	    if (!listedName.equals((String)en.nextElement())) return false;
	    if (listedName.equals(name)) {
		if (!isSelected(i)) selectAndVisible(i);
	    } else {
		if (isSelected(i)) deselect(i);
	    }
	}
	return !en.hasMoreElements();
    }

    public int listAndSelectItem(Enumeration en, String selectName) {
	clear();
	int select = -1;
	for (int i = 0; en.hasMoreElements(); i++) {
	    String name = (String)en.nextElement();
	    addItem(name);
	    if (name.equals(selectName)) select = i;
	}
	reshape();
	if (select >= 0) selectAndVisible(select);
	return select;
    }
}
