/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osam Satoh, Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;

public class DMenu extends Menu implements UseProperties {

    public DMenu(String label) {
	this(label, false);
    }

    public DMenu(String label, boolean tearOff) {
	super(label, tearOff);
	updateProperties();
    }

    public void updateProperties() {
	Font font = TeikadeProperties.getFont("font.menu");
	if (font != null && !font.equals(getFont())) {
	    setFont(font);
	}
    }

    public void add(String label) {
	add(new DMenuItem(label));
    }

    public void insert(MenuItem item, int index) {
	if (index < 0) {
	    throw new IllegalArgumentException("index less than zero.");
	}

	MenuItem[] temp = new MenuItem[countItems() - index];

	for (int i = 0; i < temp.length; i++) {
	    temp[i] = getItem(index);
	    remove(index);
	}

	add(item);

	for (int i = 0; i < temp.length; i++) {
	    add(temp[i]);
	}
    }

    public CheckboxMenuItem addCheckboxItem(String label) {
	CheckboxMenuItem item = new DCheckboxMenuItem(label);
	item.setFont(TeikadeProperties.getFont("font.menu"));
	//item.setState(true);
	add(item);
	return item;
    }

    public int indexOf(String label) {
	for (int i = countItems(); --i >= 0; ) {
	    MenuItem item = getItem(i);
	    if (item.getLabel().equals(label)) {
		return i;
	    }
	}
	return -1;
    }

    public MenuItem getItem(String label) {
	for (int i = countItems(); --i >= 0; ) {
	    MenuItem item = getItem(i);
	    if (item.getLabel().equals(label)) {
		return item;
	    }
	}
	return null;
    }

    public boolean getState(String label) {
	MenuItem item = getItem(label);
	if (item != null && (item instanceof CheckboxMenuItem)) {
	    return ((CheckboxMenuItem)item).getState();
	} else {
	    return false;
	}
    }

    public void enable(String label) {
	MenuItem item = getItem(label);
	if (item != null) {
	    item.enable();
	} else {
	    System.err.println("BUG: Could not enable MenuItem: " + label);
	}
    }

    public void disable(String label) {
	MenuItem item = getItem(label);
	if (item != null) {
	    item.disable();
	} else {
	    System.err.println("BUG: Could not disable MenuItem: " + label);
	}
    }

    public void clear() {
	for (int i = countItems(); --i >= 0; ) {
	    remove(i);
	}
    }
}
