/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.dialog;

import java.awt.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import teikade.sys.Manager;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.sys.TeikadeFile;
import teikade.sys.ProjectManager;
import teikade.util.StringSorter;
import teikade.util.Grep;
import teikade.util.CompilePropertiesEditor;
import teikade.gui.component.*;
import teikade.gui.editor.EditProperties;

public class PropertiesDialog extends TeikadeDialog {
    public static final String FONT = "Font";
    public static final String COLOR = "Color";
    public static final String LIST = "List";
    public static final String EDIT = "Edit";
    public static final String COMPILE = "Compile";
    public static final String GREP = "Grep";

    private TagCard tagCard;
    private Vector propertiesCard;
    private PropertiesDialog instance;

    public static PropertiesDialog on(Frame frame) {
	PropertiesDialog dialog = new PropertiesDialog(frame);
	dialog.init();
	return dialog;
    }

    private PropertiesDialog(Frame frame) {
	super(frame, "properties", null, OK, "apply", CANCEL);
    }

    public void init() {
	tagCard = new TagCard();
	propertiesCard = new Vector();
	add(FONT   , FontPropertiesCard.on());
	add(COLOR  , ColorPropertiesCard.on());
	add(LIST   , AttributePropertiesCard.on());
	add(EDIT   , EditPropertiesCard.on());
	add(COMPILE, CompilePropertiesCard.on());
	add(GREP   , GrepPropertiesCard.on());
	add(CENTER , tagCard);
    }

    public void add(String name, PropertiesCard card) {
	tagCard.add(name, card);
	propertiesCard.addElement(card);
    }

    public void select(String name) {
	tagCard.select(name);
    }

    public void actionMiddleButton() {
	int i = propertiesCard.size();
	while (--i >= 0) {
	    PropertiesCard card = (PropertiesCard)propertiesCard.elementAt(i);
	    String error = card.check();
	    if (error != null) {
		information(error);
		return;
	    }
	}
	i = propertiesCard.size();
	while (--i >= 0) {
	    PropertiesCard card = (PropertiesCard)propertiesCard.elementAt(i);
	    card.accept();
	}
    }

    public void actionLeftButton() {
	int i = propertiesCard.size();
	while (--i >= 0) {
	    PropertiesCard card = (PropertiesCard)propertiesCard.elementAt(i);
	    String error = card.check();
	    if (error != null) {
		information(error);
		return;
	    }
	}
	i = propertiesCard.size();
	while (--i >= 0) {
	    PropertiesCard card = (PropertiesCard)propertiesCard.elementAt(i);
	    card.accept();
	}
	SystemManager.saveProperties();
	super.actionLeftButton();
    }
}

abstract class PropertiesCard extends Panel {

    public String check() {
	return null;
    }

    public abstract void accept();
}

class FontPropertiesCard extends PropertiesCard {
    Choice sizeSelector;
    Choice codeSelector;
    Choice listSelector;
    Choice labelSelector;
    //Choice menuSelector;
    Choice dialogSelector;
    Choice defaultSelector;
    private static final String SIZE    = "font.default.size";
    private static final String CODE    = "font.code.name";
    private static final String LIST    = "font.list.name";
    private static final String LABEL   = "font.label.name";
    //private static final String MENU    = "font.menu.name";
    private static final String DIALOG  = "font.dialog.name";
    private static final String DEFAULT = "font.default.name";

    static FontPropertiesCard on() {
	FontPropertiesCard card = new FontPropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	Panel labels = new Panel();
	labels.setLayout(new GridLayout(0, 1));
	Panel selects = new Panel();
	selects.setLayout(new GridLayout(0, 1));
	sizeSelector = new DChoice();
	sizeSelector.addItem("8");
	sizeSelector.addItem("9");
	sizeSelector.addItem("10");
	sizeSelector.addItem("11");
	sizeSelector.addItem("12");
	sizeSelector.addItem("14");
	sizeSelector.addItem("16");
	sizeSelector.addItem("18");
	sizeSelector.addItem("20");
	sizeSelector.addItem("22");
	sizeSelector.addItem("24");
	String size = TeikadeProperties.getString(SIZE);
	if (size != null && size.length() > 0) {
	    sizeSelector.select(size);
	}
	labels.add(new DLabel("size:"  , Label.RIGHT));
	labels.add(new DLabel("code:"  , Label.RIGHT));
	labels.add(new DLabel("list:"  , Label.RIGHT));
	labels.add(new DLabel("label:" , Label.RIGHT));
	//labels.add(new DLabel("menu:"  , Label.RIGHT));
	labels.add(new DLabel("dialog:", Label.RIGHT));
	labels.add(new DLabel("other:" , Label.RIGHT));
	selects.add(sizeSelector);
	selects.add(codeSelector = createFontSelector(CODE));
	selects.add(listSelector = createFontSelector(LIST));
	selects.add(labelSelector = createFontSelector(LABEL));
	//selects.add(menuSelector = createFontSelector(MENU));
	selects.add(dialogSelector = createFontSelector(DIALOG));
	selects.add(defaultSelector = createFontSelector(DEFAULT));
	Panel panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add("West", labels);
	panel.add("Center", selects);
	setLayout(new BorderLayout());
	add("North", panel);
    }

    private Choice createFontSelector(String key) {
	Choice choice = new Choice();
	String[] fontNames = getToolkit().getFontList();
	for (int i = 0; i < fontNames.length; i++) {
	    choice.addItem(fontNames[i]);
	}
	if (!SystemManager.onJDK1_1 && SystemManager.onSolaris) {
	    choice.addItem("Fixed");
	}
	String selection = getFontName(key);
	choice.select(selection);
	if (!choice.getSelectedItem().equals(selection)) {
	    choice.addItem(selection);
	    choice.select(selection);
	}
	return choice;
    }

    private String getFontName(String key) {
	String name = TeikadeProperties.getString(key);
	return name != null ? name : TeikadeProperties.getString(DEFAULT);
    }

    public void accept() {
	int oldSize = TeikadeProperties.getInteger(SIZE);
	String newSize = sizeSelector.getSelectedItem();
	if (oldSize != Integer.parseInt(newSize)) {
	    TeikadeProperties.put(SIZE, newSize);
	}
	accept(codeSelector, CODE);
	accept(listSelector, LIST);
	accept(labelSelector, LABEL);
	//accept(menuSelector, MENU);
	accept(dialogSelector, DIALOG);
	accept(defaultSelector, DEFAULT);
    }

    private void accept(Choice selector, String key) {
	String name = selector.getSelectedItem();
	if (!name.equals(getFontName(key))) {
	    TeikadeProperties.put(key, name);
	}
    }
}

class ColorPropertiesCard extends PropertiesCard {
    Choice codeForeground, codeBackground;
    Choice listForeground, listBackground;
    private static final String CODE_FOREGROUND = "color.code.foreground";
    private static final String CODE_BACKGROUND = "color.code.background";
    private static final String LIST_FOREGROUND = "color.list.foreground";
    private static final String LIST_BACKGROUND = "color.list.background";
    private static final String DEFAULT = "default";

    static ColorPropertiesCard on() {
	ColorPropertiesCard card = new ColorPropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	Panel labels = new Panel();
	labels.setLayout(new GridLayout(0, 1));
	Panel colors = new Panel();
	colors.setLayout(new GridLayout(0, 2));
	labels.add(new DLabel(""));
	labels.add(new DLabel("code:"  , Label.RIGHT));
	labels.add(new DLabel("list:"  , Label.RIGHT));
	colors.add(new Label("foreground"));
	colors.add(new Label("background"));
	colors.add(codeForeground = createColorSelector(CODE_FOREGROUND));
	colors.add(codeBackground = createColorSelector(CODE_BACKGROUND));
	colors.add(listForeground = createColorSelector(LIST_FOREGROUND));
	colors.add(listBackground = createColorSelector(LIST_BACKGROUND));
	Panel pane = new Panel();
	pane.setLayout(new BorderLayout());
	pane.add("West", labels);
	pane.add("Center", colors);
	setLayout(new BorderLayout());
	add("North", pane);
    }

    private Choice createColorSelector(String key) {
	Choice choice = new Choice();
	choice.addItem(DEFAULT);
	Enumeration colorNames = StringSorter.sort(
					TeikadeProperties.getColorNames());
	while (colorNames.hasMoreElements()) {
	    choice.addItem((String)colorNames.nextElement());
	}
	String selection = getColorName(key);
	if (selection == null) {
	    choice.select(0);
	} else {
	    choice.select(selection);
	    if (!choice.getSelectedItem().equals(selection)) {
		choice.addItem(selection);
		choice.select(selection);
	    }
	}
	return choice;
    }

    private String getColorName(String key) {
	String name = TeikadeProperties.getString(key);
	return name != null ? name : DEFAULT;
    }

    public void accept() {
	accept(codeForeground, CODE_FOREGROUND);
	accept(codeBackground, CODE_BACKGROUND);
	accept(listForeground, LIST_FOREGROUND);
	accept(listBackground, LIST_BACKGROUND);
    }

    private void accept(Choice selector, String key) {
	String name = selector.getSelectedItem();
	if (!name.equals(getColorName(key))) {
	    TeikadeProperties.put(key, name);
	}
    }
}

class EditPropertiesCard extends PropertiesCard {

    private Choice keyChoice;
    private TextField softField;
    private TextField hardField;
    private Choice fileTabProject;
    private TextField fileTabValue;
    private Checkbox clipboardCheck;
    private String lastSelectedFileTabProject;
    private TextField backupField;

    private int softTab;
    private int hardTab;
    private Hashtable fileTab = new Hashtable();

    static EditPropertiesCard on() {
	EditPropertiesCard card = new EditPropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	Panel labels = new Panel();
	labels.setLayout(new GridLayout(0, 1));
	labels.add(new DLabel(    "edit key bind:", Label.RIGHT));
	labels.add(new DLabel("soft TAB width:", Label.RIGHT));
	labels.add(new DLabel("hard TAB width:", Label.RIGHT));
	labels.add(new DLabel("file TAB width:", Label.RIGHT));
	labels.add(new DLabel("use system clipboard:", Label.RIGHT));

	Panel selects = new Panel();
	selects.setLayout(new GridLayout(0, 1));
	selects.add(keyChoice = new DChoice());
	selects.add(softField = new DTextField(4));
	selects.add(hardField = new DTextField(4));
	selects.add(createFileTabPane());
	selects.add(clipboardCheck = new Checkbox());

	int num = EditProperties.countHandlers();
	for (int i = 0; i < num; i++) {
	    keyChoice.addItem(EditProperties.getHandlerName(i));
	}
	keyChoice.select(EditProperties.selectedHandler);
	softField.setText(String.valueOf(EditProperties.softTab));
	hardField.setText(String.valueOf(EditProperties.hardTab));
	hardField.setEditable(false);
	clipboardCheck.setState(EditProperties.useSystemClipboard);

	Panel backupPane = new Panel();
	backupPane.setLayout(new GridLayout(0, 1));
	backupPane.add(new DLabel("backup directory: (default: source directory)"));
	backupPane.add(backupField = new TextField(20));
	backupField.setText(Manager.getBackupDirectory());

	Panel pane = new Panel();
	pane.setLayout(new BorderLayout());
	pane.add("West", labels);
	pane.add("Center", selects);
	pane.add("South", backupPane);

	setLayout(new BorderLayout());
	add("North", pane);
    }

    protected Container createFileTabPane() {
	fileTabProject = new Choice();
	fileTabValue = new TextField(2);
	Enumeration projects = SystemManager.allProjects();
	while (projects.hasMoreElements()) {
	    String project = ((ProjectManager)projects.nextElement()).name();
	    fileTabProject.addItem(project);
	    fileTab.put(project, new Integer(EditProperties.getFileTabWidth(project)));
	}
	Panel panel = new Panel();
	panel.setLayout(new BorderLayout());
	panel.add("West", fileTabProject);
	panel.add("Center", fileTabValue);
	updateFileTabValue();
	return panel;
    }

    public boolean action(Event event, Object what) {
	if (event.target == fileTabProject) {
	    updateFileTabValue();
	    return true;
	}
	return false;
    }

    private void updateFileTabValue() {
	if (lastSelectedFileTabProject != null) {
	    try {
		fileTab.put(lastSelectedFileTabProject, Integer.valueOf(fileTabValue.getText()));
	    } catch (NumberFormatException e) {
	    }
	}
	lastSelectedFileTabProject = fileTabProject.getSelectedItem();
	fileTabValue.setText(
	    "" + ((Integer)fileTab.get(lastSelectedFileTabProject)).intValue());
    }

    public String check() {
	try {
	    softTab = Integer.parseInt(softField.getText());
	    hardTab = Integer.parseInt(hardField.getText());
	    if (softTab <= 0 || hardTab <= 0) return "Illegal number";
	} catch (NumberFormatException e) {
	    return "Illegal number format";
	}
        String dir = backupField.getText();
        if (dir != null && dir.length() > 0) {
            if (!TeikadeFile.create(dir, "").isDirectory()) {
                return "The backup directory is not a directory";
            }
        }	
	return null;
    }

    public void accept() {
	if (EditProperties.softTab != softTab) {
	    EditProperties.softTab = softTab;
	    TeikadeProperties.put("editor.tab.soft", "" + softTab);
	}
	if (EditProperties.hardTab != hardTab) {
	    EditProperties.hardTab = hardTab;
	    TeikadeProperties.put("editor.tab.hard", "" + hardTab);
	}
	int handler = keyChoice.getSelectedIndex();
	if (handler != EditProperties.selectedHandler) {
	    String name = EditProperties.selectEditKeyHandler(handler);
	    TeikadeProperties.put("editor.key", name==null ? "" : name);
	}
	updateFileTabValue();
	Enumeration keys = fileTab.keys();
	while (keys.hasMoreElements()) {
	    String project = (String)keys.nextElement();
	    int width = ((Integer)fileTab.get(project)).intValue();
	    EditProperties.setFileTabWidth(project, width);
	}
	EditProperties.useSystemClipboard(clipboardCheck.getState());
	Manager.setBackupDirectory(backupField.getText());
    }
}

class GrepPropertiesCard extends PropertiesCard {

    private Checkbox externalCheck;
    private Checkbox internalCheck;
    private TextField internalClass;

    static GrepPropertiesCard on() {
	GrepPropertiesCard card = new GrepPropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	CheckboxGroup group = new CheckboxGroup();
	externalCheck = new Checkbox("use external egrep command",
				     group, !Grep.useInternalGrep);
	internalCheck = new Checkbox("use internal egrep class",
				     group,  Grep.useInternalGrep);
	internalClass = new DTextField();
	String internalClassName = TeikadeProperties.getString(Grep.INTERNAL_CLASS_NAME);
	internalClass.setEditable(Grep.useInternalGrep);
	if (internalClassName != null) {
	    internalClass.setText(internalClassName);
	}
	Panel internalPane = new Panel();
	internalPane.setLayout(new BorderLayout());
	internalPane.add("West", internalCheck);
	Panel externalPane = new Panel();
	externalPane.setLayout(new BorderLayout());
	externalPane.add("West", externalCheck);
	Panel namePane = new Panel();
	namePane.setLayout(new BorderLayout());
	namePane.add("West", new DLabel("internal grep class name : "));
	namePane.add("Center", internalClass);
	Panel pane = new Panel();
	pane.setLayout(new GridLayout(0, 1));
	pane.add(externalPane);
	pane.add(internalPane);
	pane.add(namePane);
	pane.add(namePane);

	setLayout(new BorderLayout());
	add("North", pane);
    }

    public boolean action(Event event, Object what) {
	if (event.target == externalCheck) {
	    internalClass.setEditable(false);
	} else if (event.target == internalCheck) {
	    internalClass.setEditable(true);
	}
	return false;
    }

    public String check() {
	String grepClassName = internalClass.getText().trim();
	if (internalCheck.getState() && grepClassName.length() > 0) {
	    try {
		Class.forName(grepClassName);
	    } catch (ClassNotFoundException e) {
		return "Not found class " + grepClassName;
	    }
	}
	return null;
    }

    public void accept() {
	if (Grep.useInternalGrep != internalCheck.getState()) {
	    Grep.useInternalGrep = internalCheck.getState();
	    TeikadeProperties.put(Grep.USE_INTERNAL,
				  Grep.useInternalGrep ? "true" : "false");
	}
	if (Grep.useInternalGrep) {
	    String oldName = TeikadeProperties.getString(Grep.INTERNAL_CLASS_NAME);
	    String newName = internalClass.getText();
	    if (oldName == null && newName.length() > 0
	    || oldName != null && !oldName.equals(newName)) {
		TeikadeProperties.put(Grep.INTERNAL_CLASS_NAME, newName);
	    }
	}
    }
}

class CompilePropertiesCard extends PropertiesCard {
    CompilePropertiesEditor editor;

    static CompilePropertiesCard on() {
	CompilePropertiesCard card = new CompilePropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	editor = CompilePropertiesEditor.on(this);
    }

    public boolean action(Event event, Object what) {
	return false;
    }

    public String check() {
	return editor.check();
    }

    public void accept() {
	editor.accept();
    }
}

class AttributePropertiesCard extends PropertiesCard {

    private TextField browsingLinesField;
    private TextField listingLinesField;
    private TextField filesLinesField;

    int browsingLines;
    int listingLines;
    int filesLines;

    static AttributePropertiesCard on() {
	AttributePropertiesCard card = new AttributePropertiesCard();
	card.init();
	return card;
    }

    public void init() {
	browsingLinesField = new DTextField(4);
	listingLinesField  = new DTextField(4);
	filesLinesField    = new DTextField(4);

	Panel labels = new Panel();
	labels.setLayout(new GridLayout(3, 1));
	labels.add(new DLabel(   "browsing lines:", Label.RIGHT));
	labels.add(new DLabel(    "listing lines:", Label.RIGHT));
	labels.add(new DLabel("file dialog lines:", Label.RIGHT));

	Panel fields = new Panel();
	fields.setLayout(new GridLayout(3, 1));
	fields.add(browsingLinesField);
	fields.add(listingLinesField);
	fields.add(filesLinesField);

	Panel pane = new Panel();
	pane.setLayout(new BorderLayout());
	pane.add("West",   labels);
	pane.add("Center", fields);
	setLayout(new BorderLayout());
	add("North", pane);

	browsingLinesField.setText(TeikadeProperties.getString("attribute.browsing.lines"));
	listingLinesField.setText(TeikadeProperties.getString("attribute.listing.lines"));
	filesLinesField.setText(TeikadeProperties.getString("attribute.files.lines"));
    }

    public String check() {
	try {
	    browsingLines = Integer.parseInt(browsingLinesField.getText());
	    listingLines = Integer.parseInt(listingLinesField.getText());
	    filesLines = Integer.parseInt(filesLinesField.getText());
	    if (browsingLines < 0 || listingLines < 0 || filesLines < 0) {
		return "Illegal number";
	    }
	} catch (NumberFormatException e) {
	    return "Illegal number format";
	}
	return null;
    }

    public void accept() {
	if (TeikadeProperties.getInteger("attribute.browsing.lines") != browsingLines) {
	    TeikadeProperties.put("attribute.browsing.lines", "" + browsingLines);
	}
	if (TeikadeProperties.getInteger("attribute.listing.lines") != listingLines) {
	    TeikadeProperties.put("attribute.listing.lines", "" + listingLines);
	}
	if (TeikadeProperties.getInteger("attribute.files.lines") != filesLines) {
	    TeikadeProperties.put("attribute.files.lines", "" + filesLines);
	}
    }
}
