/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.util.Vector;
import java.util.Hashtable;
import java.util.BitSet;
import java.awt.*;
import teikade.gui.*;
import teikade.sys.SystemManager;

public class EditArea extends TextArea implements EditDataViewer {
    private EditData editData;
    private BitSet keys;		// for JDK bugs on Win32
    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    private Win32TextComponentPatch win32patch;

    public EditArea() {
	super();
	initialize();
    }

    public EditArea(int rows, int cols) {
	super(rows, cols);
	initialize();
    }

    void initialize() {
	editData = new EditData(this);
	if (SystemManager.onWin32) {
	    keys = new BitSet(256);
	    // thanks for for JDK bug patch on Win32 by Hideyo Namimoto
	    if (SystemManager.needWin32TextComponentPatch) {
		win32patch = new Win32TextComponentPatch();
	    }
	}
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public String getText() {
	String text = super.getText();
	if (editData != null) {
	    editData.update(getSelectionStart());
	}
	if (win32patch != null) win32patch.setText(text);
	return text;
    }

    public void setText(String text) {
	super.setText(text);
	// thanks for JDK bug patch on Win32 by Hideyo Namimoto
	if (win32patch != null) win32patch.setText(text);
	editData.reset();
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public void appendText(String text) {
        super.appendText(text);
        if (win32patch != null) getText();
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public void insertText(String text, int pos) {
        super.insertText(text, pos);
        if (win32patch != null) getText();
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public void replaceText(String text, int start, int end) {
        super.replaceText(text, start, end);
        if (win32patch != null) getText();
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public void select(int start, int end) {
	if (win32patch != null) {
	    if (start == end) {
		int n = win32patch.adjustPoint(start);
		start += n;
		end   += n;
	    } else {
		start += win32patch.adjustRange(start);
		end   += win32patch.adjustRange(end);
	    }
	}
	super.select(start, end);
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public int getSelectionStart() {
	int start = super.getSelectionStart();
	if (win32patch != null) start -= win32patch.adjustForGetSelection(start);
	return start;
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public int getSelectionEnd()  {
	int end = super.getSelectionEnd();
	if (win32patch != null) end -= win32patch.adjustForGetSelection(end);
	return end;
    }

    // thanks for JDK bug patch on Win32 by Hideyo Namimoto
    public boolean mouseDown(Event evt, int x, int y) {
	if (editData != null) {
	  editData.update();
	}
	return super.mouseDown(evt,x,y);
    }

    public EditData getEditData() {
	return editData;
    }

    public boolean keyDown(Event event, int key) {
	if (editData != null) {
	    EditKeyHandler keyHandler = EditProperties.getKeyHandler(isEditable());
	    if (keyHandler != null) {
		editData.update();
		editData.pushStatus();
		if (keys != null && event.key > 0 && event.key < 256) {
		    keys.set(event.key);
		}
		if (win32patch != null) win32patch.patchEvent(event);
		return keyHandler.handleKey(editData, event);
	    }
	}
	return super.keyDown(event, key);
    }

    public boolean keyUp(Event event, int key) {
	if (keys != null && editData != null
	&&  event.key > 0 && event.key < 256
	&&  EditProperties.getKeyHandler(isEditable()) != null) {
	    if ((event.modifiers & Event.CTRL_MASK) != 0
	    &&  !keys.get(event.key)) {
		keyDown(event, key);
	    }
	    keys.clear(event.key);
	    return true;
	}
	return super.keyUp(event, key);
    }

    public int displayLines() {
	return size().height / getFont().getSize();
    }
}
