/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.awt.*;
import teikade.gui.TeikadeFrame;
import teikade.gui.Transcript;

public class EmacsEditKeyHandler extends EmacsViewKeyHandler {
    private static final String WORD = "word";
    private static final String NONWORD = "nonword";

    public boolean withNone(EditData editData, Event event) {
	if (super.withNone(editData, event)) return true;
	boolean shift = event.shiftDown();
	switch (event.key) {
	case '\t':
	    editData.autoIndent();
	    //editData.tab(shift);
	    return true;
	case '\n':
	    editData.insert("\n").autoIndent().packUndo();
	    return true;
	case '\b':
	    if (!editData.cutSelectedText()) {
		editData.backSpace();
	    }
	    return true;
	case '\177':
	    if (!editData.cutSelectedText()) {
		editData.anchor().forwardChar().delete();
	    }
	    return true;
	case 1025:
	    editData.paste();
	    return true;
	default:
	    if (event.key >= ' ' && event.key < 127) {
		editData.insert("" + (char)event.key);
		if (event.key == '}'
		||  event.key == ':' && editData.startsWithCaseOrDefault()) {
		    editData.autoIndent();
		} else {
		    String s = Character.isJavaLetterOrDigit((char)event.key) ? WORD: NONWORD;
		    if (editData.lastStatus == s) editData.packUndo();
		    editData.setStatus(null, s);
		}
		return true;
	    }
	}
	return false;
    }

    public boolean withCtrl(EditData editData, Event event) {
	switch (event.key) {
	case '/':
	case '_': case '\037':
	    editData.undo();
	    return true;
	case 'd': case '\004': case '\177':
	    editData.anchor().forwardChar().delete();
	    return true;
	case 'h': case '\010':
	    if (!super.withCtrl(editData, event)) {
		editData.backSpace();
	    }
	    return true;
	case 'i': case '\011':
	    editData.autoIndent();
	    return true;
	case 'j': case '\012':
	    editData.insert("\n").autoIndent().packUndo();
	    return true;
	case 'k': case '\013':
	    if (editData.onEndOfLine()) {
		editData.anchor().forwardChar().kill();
	    } else {
		editData.anchor().endOfLine().kill();
	    }
	    return true;
	case 'm': case '\015':
	    editData.insert("\n");
	    return true;
	case 'o': case '\017':
	    editData.insert("\n").backwardChar().move();
	    return true;
	case 'w': case '\027':
	    if (editData.anchorAtMark().kill() == null) {
		editData.setStatus("The mark is not set now");
	    }
	    return true;
	case 'y': case '\031':
	    editData.paste();
	    return true;
	}
	return super.withCtrl(editData, event);
    }

    public boolean withCtrlX(EditData editData, Event event) {
	switch (event.key) {
	case 's':
	    editData.setStatus("Accepting... ");
	    Thread.yield();
	    if (editData.notifyObserver(EditDataObserver.ACCEPT)
	     && editData.notifyObserver(EditDataObserver.SYNC)) {
		editData.setStatus("Accepting... done.");
	    } else {
		editData.setStatus("Accepting... aborted.");
	    }
	    return true;
	}
	return super.withCtrlX(editData, event);
    }

    public boolean withCtrlXCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 's': case '\023':
	    editData.setStatus("Accepting... ");
	    Thread.yield();
	    if (editData.notifyObserver(EditDataObserver.ACCEPT)) {
		editData.setStatus("Accepting... done.");
	    } else {
		editData.setStatus("Accepting... aborted.");
	    }
	    return true;
	case 'v': case '\026':
	    return editData.notifyObserver(EditDataObserver.RESET);
	case 'w': case '\027':
	    editData.setStatus("Syncing... ");
	    Thread.yield();
	    if (editData.notifyObserver(EditDataObserver.ACCEPT)
	     && editData.notifyObserver(EditDataObserver.SYNC)) {
		editData.setStatus("Syncing... done.");
	    } else {
		editData.setStatus("Syncing... aborted.");
	    }
	    return true;
	}
	return super.withCtrlXCtrl(editData, event);
    }

    public boolean withCtrlCCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 'c': case '\003':
	    return editData.notifyObserverAsynchronous(EditDataObserver.COMPILE);
	}
	return super.withCtrlCCtrl(editData, event);
    }

    public boolean withMeta(EditData editData, Event event) {
	switch (event.key) {
	case '\\': 
	    editData.beginningOfSpaces().anchor().endOfSpaces().cut();
	    return true;
	case ' ': 
	    editData.beginningOfSpaces().anchor().endOfSpaces().cut().insert(" ");
	    return true;
	case 'd': case '\004':
	    editData.anchor().endOfUnLetterOrDigit().endOfLetterOrDigit().kill();
	    return true;
	case '\010':
	    editData.anchor().beginningOfUnLetterOrDigit().beginningOfLetterOrDigit().kill();
	    return true;
	case 'i': case '\t':
	    editData.insert("\t");
	    return true;
	}
	return super.withMeta(editData, event);
    }

    public boolean withMetaCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 'i': case '\t':
	    editData.insert("\t");
	    return true;
	case 'k': case '\013':
	    try {
		editData.anchor().forwardExpression().kill();
	    } catch (NullPointerException e) {
	    }
	    return true;
	case 'o': case '\017':
	    String spaces = editData.spacesForBOL(editData.getColumn());
	    editData.anchor().insert("\n" + spaces).moveToAnchor();
	    return true;
	case 'w': case '\027':
	    if (editData.anchorAtMark().appendKill() == null) {
		editData.setStatus("The mark is not set now");
	    }
	    return true;
	}
	return super.withMetaCtrl(editData, event);
    }
}
