/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.awt.*;
import teikade.gui.TeikadeFrame;
import teikade.gui.Transcript;
import teikade.sys.SystemManager;

class EmacsViewKeyHandler extends SimpleViewKeyHandler {
    private static final String AfterMeta = "M-";
    private static final String AfterCtrlX = "C-x-";
    private static final String AfterCtrlC = "C-c-";
    private static final String AfterCtrlU = "C-u-";
    private static final String ForwardSearch = "I-search: ";
    private static final String BackwardSearch = "I-search backward: ";

    public boolean handleKey(EditData editData, Event event) {
	if (!handleEmacsKey(editData, event)) {
	    if (SystemManager.onJDK1_1) {
		// pass through FEP on/off key etc.
		if (event.key == 0 || event.key == 28 // special or Ctrl+\  
		 || (SystemManager.needWin32TextComponentPatch
		     && event.key >= Event.HOME && event.key <= Event.RIGHT)) {
		    return false;
		}
	    }
	    editData.setStatus("undefined");
	}
	return true;	//cancel
    }

    boolean handleEmacsKey(EditData editData, Event event) {
	if ((event.modifiers & ~Event.SHIFT_MASK) == Event.CTRL_MASK
	&&  (event.key == 'g' || event.key == '\007')) {
	    if (editData.lastStatus == ForwardSearch
	    ||  editData.lastStatus == BackwardSearch) {
		editData.quitSearch().move();
	    }
	    editData.setStatus("Quit");
	    return true;
	}
	if (editData.lastStatus == AfterMeta) {
	    event.modifiers |= Event.META_MASK;
	}
	boolean control = event.controlDown();
	boolean meta = event.metaDown();
	if (editData.lastStatus == AfterCtrlX) {
	    if (!control && !meta) {
		return withCtrlX(editData, event);
	    } else if (control && !meta) {
		return withCtrlXCtrl(editData, event);
	    } else if (!control && meta) {
		return withMetaCtrlX(editData, event);
	    } else {
		return withMetaCtrlXCtrl(editData, event);
	    }
	}
	if (editData.lastStatus == AfterCtrlC) {
	    if (!control && !meta) {
		return withCtrlC(editData, event);
	    } else if (control && !meta) {
		return withCtrlCCtrl(editData, event);
	    } else if (!control && meta) {
		return withMetaCtrlC(editData, event);
	    } else {
		return withMetaCtrlCCtrl(editData, event);
	    }
	}
	return super.handleKey(editData, event);
    }

    public boolean withNone(EditData editData, Event event) {
	switch (event.key) {
	case '\b':
	    if (editData.lastStatus == ForwardSearch
	    ||  editData.lastStatus == BackwardSearch) {
		editData.backspaceSearchString().move();
		editData.setStatus(editData.lastStatus + editData.searchString,
				   editData.lastStatus);
		return true;
	    }
	    break;
	case '\033':
	    editData.setStatus(AfterMeta, AfterMeta);
	    return true;
	}
	if (super.withNone(editData, event)) {
	    return true;
	}
	char key = (char)event.key;
	if ((int)key == event.key && key >= ' ' && Character.isDefined(key)) {
	    return insert(editData, "" + key);
	}
	return false;
    }

    public boolean withCtrl(EditData editData, Event event) {
	switch (event.key) {
	case ' ': 
	case '@': case '\000':
	    editData.setMark();
	    return true;
	case '[': case '\033':
	    editData.setStatus(AfterMeta, AfterMeta);
	    return true;
    	case 'a': case '\001':
	    editData.beginningOfLine().move();
	    return true;
	case 'b': case '\002':
	    if (editData.onBeginningOfText()) {
		editData.setStatus("Beginning of text");
	    } else {
		editData.backwardChar().move();
	    }
	    return true;
	case 'c': case '\003':
	    editData.setStatus(AfterCtrlC, AfterCtrlC);
	    return true;
	case 'e': case '\005':
	    editData.endOfLine().move();
	    return true;
	case 'f': case '\006':
	    if (editData.onEndOfText()) {
		editData.setStatus("End of text");
	    } else {
		editData.forwardChar().move();
	    }
	    return true;
	case 'h': case '\010':
	    if (editData.lastStatus == ForwardSearch
	    ||  editData.lastStatus == BackwardSearch) {
		editData.backspaceSearchString().move();
		editData.setStatus(editData.lastStatus + editData.searchString,
				   editData.lastStatus);
		return true;
	    }
	    break;
	case 'n': case '\016':
	    editData.nextLine();
	    return true;
	case 'p': case '\020':
	    editData.previousLine();
	    return true;
	case 'r': case '\022':
	    if (editData.lastStatus == BackwardSearch) {
		if (editData.searchString.length() == 0) {
		    editData.setLastSearchString();
		} else {
		    editData.addSearchString("");
		}
	    } else if (editData.lastStatus != ForwardSearch) {
		editData.clearSearchString();
		editData.setStatus(BackwardSearch, BackwardSearch);
		return true;
	    }
	    editData.lastStatus = BackwardSearch;
	    insert(editData, null);
	    return true;
	case 's': case '\023':
	    if (editData.lastStatus == ForwardSearch) {
		if (editData.searchString.length() == 0) {
		    editData.setLastSearchString();
		} else {
		    editData.addSearchString("");
		}
	    } else if (editData.lastStatus != BackwardSearch) {
		editData.clearSearchString();
		editData.setStatus(ForwardSearch, ForwardSearch);
		return true;
	    }
	    editData.lastStatus = ForwardSearch;
	    insert(editData, null);
	    return true;
	case 'v': case '\026':
	    editData.nextPage().move();
	    return true;
	case 'x': case '\030':
	    editData.setStatus(AfterCtrlX, AfterCtrlX);
	    return true;
	}
	return super.withCtrl(editData, event);
    }

    public boolean withMeta(EditData editData, Event event) {
	switch (event.key) {
	case '<':
	    editData.setMark().beginningOfText().move();
	    return true;
	case '>':
	    editData.setMark().endOfText().move();
	    return true;
	case 'b': case '\002':
	    editData.beginningOfUnLetterOrDigit().beginningOfLetterOrDigit().move();
	    return true;
	case 'f': case '\006':
	    editData.endOfUnLetterOrDigit().endOfLetterOrDigit().move();
	    return true;
	case 'v': case '\026':
	    editData.previousPage().move();
	    return true;
	case 'w': case '\027':
	    if (editData.anchorAtMark().copy() == null) {
		editData.setStatus("The mark is not set now");
	    }
	    return true;
	}
	return super.withMeta(editData, event);
    }

    public boolean withMetaCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 'n': case '\016':
	    try {
		editData.forwardPair().move();
	    } catch (NullPointerException e) {
	    }
	    return true;
	case 'p': case '\020':
	    try {
		editData.backwardPair().move();
	    } catch (NullPointerException e) {
	    }
	    return true;
	}
	return false;
    }

    public boolean withCtrlX(EditData editData, Event event) {
	switch (event.key) {
	case '0':
	    return editData.notifyObserver(EditDataObserver.CLOSE);
	case '2':
	    return editData.notifyObserver(EditDataObserver.SPAWN);
	case '5':
	    return editData.notifyObserver(EditDataObserver.SPAWNNARROW);
	case 'h':
	    editData.endOfText().setMark().beginningOfText().move();
	    return true;
	}
	return false;
    }

    public boolean withCtrlXCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 'c': case '\003':
	    return editData.notifyObserver(EditDataObserver.QUIT);
	case 'x': case '\030':
	    editData.swapWithMark().move();
	    return true;
	}
	return false;
    }

    public boolean withMetaCtrlX(EditData editData, Event event) {
	return false;
    }

    public boolean withMetaCtrlXCtrl(EditData editData, Event event) {
	return false;
    }

    public boolean withCtrlC(EditData editData, Event event) {
	return false;
    }

    public boolean withCtrlCCtrl(EditData editData, Event event) {
	switch (event.key) {
	case 'e': case '\005':
	    return editData.notifyObserverAsynchronous(EditDataObserver.EXECUTE);
	}
	return false;
    }

    public boolean withMetaCtrlC(EditData editData, Event event) {
	return false;
    }

    public boolean withMetaCtrlCCtrl(EditData editData, Event event) {
	return false;
    }

    public boolean insert(EditData editData, String text) {
	if (editData.lastStatus == ForwardSearch) {
	    if (text != null) editData.addSearchString(text);
	    try {
		editData.forwardSearch().move();
		editData.setStatus(ForwardSearch + editData.searchString,
				   ForwardSearch);
	    } catch (NullPointerException e) {
		editData.setStatus("Failing " + ForwardSearch +
				   editData.searchString, ForwardSearch);
	    }
	    return true;
	}
	if (editData.lastStatus == BackwardSearch) {
	    if (text != null) editData.addSearchString(text);
	    try {
		editData.backwardSearch().move();
		editData.setStatus(BackwardSearch + editData.searchString,
				   BackwardSearch);
	    } catch (NullPointerException e) {
		editData.setStatus("Failing " + BackwardSearch +
				   editData.searchString, BackwardSearch);
	    }
	    return true;
	}
	return false;
    }
}
