/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.awt.Event;
import teikade.sys.SystemManager;

class SimpleViewKeyHandler implements EditKeyHandler {

    public boolean handleKey(EditData editData, Event event) {
	boolean control = event.controlDown();
	boolean meta = event.metaDown();
	if (!control && !meta) {
	    return withNone(editData, event);
	} else if (control && !meta) {
	    return withCtrl(editData, event);
	} else if (!control && meta) {
	    return withMeta(editData, event);
	} else {
	    return withMetaCtrl(editData, event);
	}
    }

    public boolean withNone(EditData editData, Event event) {
	boolean shift = event.shiftDown();
	if (shift && SystemManager.needWin32TextComponentPatch
	 && event.key > Event.HOME && event.key <= Event.RIGHT) {
	    return false;
	}
	int targetColumn = -1;
	switch (event.key) {
	case Event.UP:
	    targetColumn = editData.getTargetColumn();
	    editData.anchorAtSelection().previousLine();
	    break;
	case Event.DOWN:
	    targetColumn = editData.getTargetColumn();
	    editData.anchorAtSelection().nextLine();
	    break;
	case Event.RIGHT:
	    editData.anchorAtSelection().forwardChar();
	    break;
	case Event.LEFT:
	    editData.anchorAtSelection().backwardChar();
	    break;
	case Event.HOME:
	    editData.anchorAtSelection().beginningOfLine();
	    break;
	case Event.END:
	    editData.anchorAtSelection().endOfLine();
	    break;
	case Event.PGUP:
	    editData.anchorAtSelection().previousPage();
	    break;
	case Event.PGDN:
	    editData.anchorAtSelection().nextPage();
	    break;
	default:
	    return false;
	}
	if (shift) {
	    editData.select();
	} else {
	    editData.move();
	}
	if (targetColumn >= 0) editData.setTargetColumn(targetColumn);
	return true;
    }

    public boolean withCtrl(EditData editData, Event event) {
	boolean shift = event.shiftDown();
	switch (event.key) {
	case Event.RIGHT:
	    if (!shift) return false;
	    editData.anchorAtSelection().endOfUnLetterOrDigit().endOfLetterOrDigit().select();
	    return true;
	case Event.LEFT:
	    if (!shift) return false;
	    editData.anchorAtSelection().beginningOfUnLetterOrDigit().beginningOfLetterOrDigit().select();
	    return true;
	case Event.HOME:
	    if (!shift) return false;
	    editData.anchorAtSelection().beginningOfText().select();
	    return true;
	case Event.END:
	    if (!shift) return false;
	    editData.anchorAtSelection().endOfText().select();
	    return true;
	}
	return false;
    }

    public boolean withMeta(EditData editData, Event event) {
	return false;
    }

    public boolean withMetaCtrl(EditData editData, Event event) {
	return false;
    }
}
