/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.util.Vector;

public class UndoBuffer {
    private EditData editData;
    private Vector undoBuffer;
    private int undoPoint;
    private boolean undoing;
    private boolean packed;

    public UndoBuffer(EditData editData) {
	this.editData = editData;
	undoBuffer = new Vector();
	initialize();
    }

    private void initialize() {
	undoPoint = 0;
	undoing = false;
	packed = false;
    }

    public void clear() {
	undoBuffer.removeAllElements();
	initialize();
    }

    public void push(int pos, String oldText, String newText) {
	if (packed) {
	    UndoData undo0 = (UndoData)undoBuffer.elementAt(0);
	    undo0.pos = pos;
	    undo0.oldText = oldText;
	    undo0.newText = newText;
	    packed = false;
	} else {
	    undoBuffer.insertElementAt(new UndoData(pos, oldText, newText), 0);
	    if (undoing) undoPoint++;
	}
	if (!undoing) undoPoint = 0;
	if (undoBuffer.size() > 300) {
	    undoBuffer.setSize(300);
	    if (undoPoint >= 300) undoPoint = 300;
	}
    }

    public void resetPoint() {
	if (!undoing) undoPoint = 0;
    }

    public boolean undo() {
	if (undoPoint == 0 && packed) undoPoint++;
	if (undoPoint >= undoBuffer.size()) {
	    undoPoint = 0;
	    return false;
	}
	undoing = true;
	UndoData data = (UndoData)undoBuffer.elementAt(undoPoint);
	editData.replaceText(data.oldText,
			 data.pos, data.pos + data.newText.length());
	editData.select(data.pos + data.oldText.length());
	undoPoint++;
	undoing = false;
	return true;
    }

    public void pack() {
	if (packed || undoBuffer.size() < 2) return;
	UndoData undo0 = (UndoData)undoBuffer.elementAt(0);
	UndoData undo1 = (UndoData)undoBuffer.elementAt(1);
	if (undo1.pos + undo1.newText.length() == undo0.pos) {
	    undo1.oldText += undo0.oldText;
	    undo1.newText += undo0.newText;
	    packed = true;
	}
    }
}

class UndoData {
    int pos;
    String oldText;
    String newText;

    UndoData(int pos, String oldText, String newText) {
	this.pos = pos;
	this.oldText = oldText;
	this.newText = newText;
    }
}
