package teikade.gui.menu;

import java.awt.*;
import java.util.Enumeration;
import teikade.sys.*;
import teikade.gui.*;
import teikade.gui.component.*;
import teikade.gui.dialog.MethodOrder;
import teikade.lang.AccessFlags;
import teikade.lang.MethodPrototype;

public class HierarchyMenuBar extends TeikadeMenuBar {

    private DMenu optionMenu;

    protected final static String MENU_GO_TO_LINE      = "go to line";
    protected final static String MENU_SPAWN_CHANGES   = "spawn changes";
    protected final static String MENU_SPAWN_FILE      = "spawn file";
    protected final static String MENU_ORDER           = "order";
    protected final static String MENU_CHANGE_ORDER    = "change order";

    protected final static String MENU_HIDE_CLASS_METHODS     = "hide class methods";
    protected final static String MENU_HIDE_INSTANCE_METHODS  = "hide instance methods";
    protected final static String MENU_HIDE_PUBLIC_METHODS    = "hide public methods";
    protected final static String MENU_HIDE_FRIENDLY_METHODS  = "hide friendly methods";
    protected final static String MENU_HIDE_PROTECTED_METHODS = "hide protected methods";
    protected final static String MENU_HIDE_PRIVATE_METHODS   = "hide private methods";

    public HierarchyMenuBar(DebugMenu debugMenu) {
	super(debugMenu);
    }

    protected DMenu createOptionMenu() {
	DMenu menu = super.createOptionMenu();
	menu.addSeparator();
	menu.addCheckboxItem(MENU_HIDE_CLASS_METHODS);
	menu.addCheckboxItem(MENU_HIDE_INSTANCE_METHODS);
	menu.addSeparator();
	menu.addCheckboxItem(MENU_HIDE_PUBLIC_METHODS);
	menu.addCheckboxItem(MENU_HIDE_FRIENDLY_METHODS);
	menu.addCheckboxItem(MENU_HIDE_PROTECTED_METHODS);
	menu.addCheckboxItem(MENU_HIDE_PRIVATE_METHODS);
	optionMenu = menu;
	return menu;
    }

    protected void createFileEditMenu(DMenu menu) {
        menu.add(MENU_GO_TO_LINE);
        menu.addSeparator();
        menu.add(MENU_SPAWN_CHANGES);
        menu.add(MENU_SPAWN_FILE);
        menu.addSeparator();
    }

    protected void createPackageEditMenu(DMenu menu) {
    }

    protected void createClassEditMenu(DMenu menu) {
    }

    protected void createMethodEditMenu(DMenu menu) {
	menu.add(new MethodOrderMenu(MENU_ORDER));
	menu.add(MENU_CHANGE_ORDER);
	menu.addSeparator();
    }

    protected void updateFileMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.classManager() != null || cp.methodManager() != null) {
	    menu.enable(MENU_SPAWN_FILE);
	} else {
	    menu.disable(MENU_SPAWN_FILE);
	}
	super.updateFileMenuActivity(menu, frame, cp);
    }

    protected void updatePackageMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	super.updatePackageMenuActivity(menu, frame, cp);
    }

    protected void updateClassMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	super.updateClassMenuActivity(menu, frame, cp);
    }

    protected void updateMethodMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	ClassManager cm = cp.classManager();
	if (cm == null) {
	    menu.disable(MENU_ORDER);
	    menu.disable(MENU_CHANGE_ORDER);
	} else {
	    boolean isInterface = false;
	    try {
		if (cm.spec().isInterface()) isInterface = true;
	    }
	    catch (TeikadeException e) {
	    }
	    if (isInterface) {
		menu.disable(MENU_ORDER);
		menu.disable(MENU_CHANGE_ORDER);
	    } else {
		menu.enable(MENU_ORDER);
		menu.enable(MENU_CHANGE_ORDER);
	    }
	}
	super.updateMethodMenuActivity(menu, frame, cp);
    }

    protected void actionFileMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_GO_TO_LINE)) {
	    (new LocationFrame(cp)).open();
        } else if (label.equals(MENU_SPAWN_CHANGES)) {
            ChangeListFrame.openOn(cp);
        } else if (label.equals(MENU_SPAWN_FILE)) {
	    SourceFileManager sfm = cp.sourceFileManager();
            (new TeikadeFrame(new CodePath(sfm), "Source File '" + sfm.name() + "'")).open();
	} else {
	    super.actionFileMenu(label, frame, cp);
	}
    }

    protected void actionOptionMenu(String label, TeikadeFrame frame, CodePath cp) {
	try {
	    ((HierarchyBrowser)frame).updateMethods();
	}
	catch (TeikadeException e) {
	}
	super.actionOptionMenu(label, frame, cp);
    }

    protected void actionPackageMenu(String label, TeikadeFrame frame, CodePath cp) {
	    super.actionPackageMenu(label, frame, cp);
    }

    protected void actionClassMenu(String label, TeikadeFrame frame, CodePath cp) {
	    super.actionClassMenu(label, frame, cp);
    }

    protected void actionMethodMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_CHANGE_ORDER)) {
	    (new MethodOrder(frame, cp)).show();
	} else {
	    super.actionMethodMenu(label, frame, cp);
	}
    }

    public boolean action(Event e, Object arg, TeikadeFrame frame, CodePath cp)
					throws TeikadeNoClassException {
	return super.action(e, arg, frame, cp);
    }

    public AccessFlags hideMethods() {
	AccessFlags hideMethods = new AccessFlags();
	if (optionMenu.getState(MENU_HIDE_CLASS_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.STATIC);
	}
	if (optionMenu.getState(MENU_HIDE_INSTANCE_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.NON_STATIC);
	}
	if (optionMenu.getState(MENU_HIDE_PUBLIC_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PUBLIC);
	}
	if (optionMenu.getState(MENU_HIDE_FRIENDLY_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.FRIENDLY);
	}
	if (optionMenu.getState(MENU_HIDE_PROTECTED_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PROTECTED);
	}
	if (optionMenu.getState(MENU_HIDE_PRIVATE_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PRIVATE);
	}
	return hideMethods;
    }
}
