/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.sys;

import teikade.lang.*;
import teikade.util.*;
import java.io.*;
import java.util.*;

public final class ClassManager extends Manager {
    ClassSpec spec;

    /**
     * holds Vector of SourceCodeRange or StringSourceCode or Changes)
     */
    Object code = null;

    /**
     * holds SourceCodeRange for this class
     */
    SourceCodeRange range = null;

    /**
     * holds super class
     */
    ClassManager superClass = null;
    String unknownSuperClass = null;

    /**
     * id number for execute
     */
    private static int idNum = 0;

    private static boolean canExecute = false;
    private static String java = null;
    private static String javapm = null;

    static {
	String bin = System.getProperty("java.home") +
		     File.separator + "bin" + File.separator;
	if (SystemManager.onWin32 || SystemManager.onOS2) {
	    java = bin + "java.exe";
	    javapm = bin + "javapm.exe";
	} else {
	    java = bin + "java";
	}
	if ((new File(java)).exists()) {
	    canExecute = true;
	} else {
	    if ("true".equals(System.getProperty("debug.execute"))) {
		System.err.println("debug.execute: not found " + java);
	    }
	}
    }

    public static boolean canExecute(){ 
	return canExecute;
    }

    /**
     * resolve dependancy of ClassSpec
     */
    public boolean resolveDependancy() {
	return resolveDependancy(spec);
    }

    public ClassManager(Manager parent, ClassSpec spec) {
        super(parent, spec.className());
        this.spec = spec;
    }

    /**
     * read a class description via CodeReader
     * @return ClassManager for the description
     */
    synchronized
    static ClassManager readFrom(SourceFileManager parent, CodeReader reader) throws DescriptionException {
	//try {
	    Vector ranges = new Vector();
	    TeikadeFile file = parent.file();
	    ClassSpec spec = new ClassSpec(reader.description());
	    ClassManager cman = new ClassManager(parent, spec);
	    cman.addChild(new HeaderManager(cman));
	    int firstStartLine = reader.startLine();
	    int startPoint = reader.startPoint();
	    int startLine  = reader.startLine();
	    int endPoint = startPoint;
	    int endLine = startLine -1;
	    int nest;
	    while ((nest = reader.nextDescription()) >= 1) {
		endPoint = reader.startPoint();
		endLine  = reader.startLine() - 1;
		MethodManager mman = MethodManager.readFrom(cman, reader, nest);
		if (mman != null) {
		    cman.addChild(mman);
		    if (startPoint != -1) {
			ranges.addElement(
			    new SourceCodeRange(startPoint, endPoint,
						startLine, endLine));
		    }
		    startPoint = -1;
		    startLine = -1;
		} else if (startPoint == -1) {
		    startPoint = endPoint;
		    startLine = endLine + 1;
		}
	    }
	    if (startPoint == -1) {
		startPoint = reader.startPoint();
		startLine = reader.startLine();
	    }
	    endPoint = reader.endPoint();
	    endLine  = reader.endLine();
	    ranges.addElement(new SourceCodeRange(startPoint, endPoint,
						  startLine, endLine));
	    cman.code = ranges;
	    cman.range = new SourceCodeRange(-1, -1, firstStartLine, endLine);
	    return cman;
	//} catch (DescriptionException e) {
	    //SystemManager.exception(parent.absolutePath() + ": " + reader.startLine(), e);
	    //return null;
	//}
    }

    public ClassManager(Manager parent, String name) {
	super(parent, name);
	spec = null;
	code  = null;
    }

    ProjectManager projectManager() {
	return (ProjectManager)parent().parent().parent();
    }

    PackageManager packageManager() {
	return (PackageManager)parent().parent();
    }

    /**
     * returns super class for me
     * resolve package name using header information
     */
    public ClassManager superClass() throws TeikadeException {
	superClass = null;
	readSource();
	if (spec.isInterface()) {
	    return null;	// has no super class
	}
	String sc = spec.superClassName();
	if (sc == null) {
	    return null;	// the super class is java.lang.Object
	}
	int dot = sc.lastIndexOf('.');
	if (dot > 0) {
	    // the super class name contains a package name
	    PackageManager pm = PackageManager.findPackage(sc.substring(0, dot));
	    if (pm != null
	     && (superClass = pm.findClass(sc.substring(dot + 1))) != null) {
		return superClass;
	    }
	} else if ((superClass = packageManager().findClass(sc)) != null) {
	    return superClass;
	} else {
	    // not found the super class in current package
	    Vector imported = header().imported();
	    imported.addElement(PackageManager.findPackage("java.lang"));
	    Enumeration m = imported.elements();
	    while (m.hasMoreElements()) {
		Object man = m.nextElement();
		if (man instanceof ClassManager) {
		    if (((ClassManager)man).name().equals(sc)) {
			superClass = (ClassManager)man;
			return superClass;
		    }
		} else if (man instanceof PackageManager) {
		    superClass = ((PackageManager)man).findClass(sc);
		    if (superClass != null) {
			return superClass;
		    }
		}
	    }
	}
	// not found the super class
	if (unknownSuperClass == null || !unknownSuperClass.equals(sc)) {
	    SystemManager.information("Not found super class " + sc + " for class " + name + " .");
	    unknownSuperClass = sc;
	}
	return null;
    }

    /**
     * get HeaderManager for this class
     *  -> the first child is it
     */
    HeaderManager header() {
	if (child == null) {
	    addChild(new HeaderManager(this));
	}
	return (HeaderManager)child;
    }

    /**
     * get hierarchy list of this class
     */
    public Enumeration hierarchy(String methodName) {
	return new HierarchyEnumerator(this, methodName);
    }

    /**
     * get binary file for this class
     */
    TeikadeFile binaryFile(TeikadeFile directory) {
	String classPathProperty = classPathProperty();
	if (classPathProperty != null) {
	    directory = TeikadeFile.create(classPathProperty, (header().packageName() == null) ? "" : packageManager().path);
	}
	return TeikadeFile.create(directory, name + ".class");
    }

    /**
     * get binary file for this class
     */
    TeikadeFile binaryPath() {
	String classPathProperty = classPathProperty();
	if (classPathProperty != null) {
	    return TeikadeFile.create(classPathProperty, (header().packageName() == null) ? "" : packageManager().path);
	} else {
	    return ((SourceFileManager)parent).directory();
	}
    }

    /**
     * get class path for this class
     */
    String classPathProperty() {
	String projectName = projectManager().name();
	return Compile.getInstance(projectName)
		      .getCompileProperties()
		      .getParsedString("compile.class.path." + projectName);
    }

    /**
     * get class CodePath
     */
    public CodePath getCodePath() {
	if (parent != null) {
	    CodePath cp = parent.getCodePath();
	    cp.classManager(this);
	    return cp;
	} else {
	    return new CodePath(null, null, null, this, null);
	}
    }

    /**
     * whether this class is editable or not
     */
    public boolean isEditable() {
	if (parent != null) {
	    return parent.isEditable();
	} else {
	    return false;
	}
    }

    /**
     * whether this class is updated after last compilation
     */
    boolean isUpdatedAfterLastCompilation(long sourceFileLastModified, TeikadeFile directory) {
	if (isChanged()) return true;
	TeikadeFile binaryFile = binaryFile(directory);
	return !binaryFile.exists() ||
	    (binaryFile.lastModified() < sourceFileLastModified);
    }

    /**
     * set new code for this class
     */
    public Manager setCode(String newCode)
    throws TeikadeException {
	String description = "";
	ClassSpec newSpec;
	CodeReader reader = new CodeReader(newCode);
	int nest = reader.nextDescription();
	if (nest <= 0) {	// -1:EOF 0:abstruct..? 1:class description
	    return tryForMethod(newCode);
	}
	description = reader.description();
	if (nest > 0) reader.nextDescription(false);
	if (reader.nextDescription() >= 0) {
	    throw new DescriptionException("too many description or not enough open brace");
	}
	try {
	    newSpec = new ClassSpec(description);
	} catch (DescriptionException ce) {
	    String errorMessage = ce.getMessage();
	    if (errorMessage.equals(ClassSpec.NO_CLASS_OR_INTERFACE) ||
		description.indexOf('(') >= 0 ||
		description.indexOf(')') >= 0) {
		return tryForMethod(newCode);
	    }
	    throw new DescriptionException(errorMessage);
	}
	if (!newSpec.className().equals(spec.className())) {
	    PackageManager pman = packageManager();
	    return pman.createClass(newSpec, newCode);
	}
	Manager ret = changed(newCode);
	if (!resolveDependancy(newSpec)) return null;
	spec = newSpec;
	return ret;
    }

    /**
     * check for method description and proceed if it is
     */
    private Manager tryForMethod(String newCode)
    throws TeikadeException {
	return child.setCode(newCode);
    }

    /**
     * resolve dependancy of ClassSpec
     */
    private boolean resolveDependancy(ClassSpec newSpec) {
	String className = newSpec.superClassName();
	Vector imported = header().imported();
	Vector imports = new Vector();
	if (className != null) {
	    if (newSpec.isInterface()) {
		StringTokenizer tokenizer =
		    new StringTokenizer(className + " ", ", \t\n\r");
		while (tokenizer.hasMoreTokens()) {
		    if (!resolveDependancy(tokenizer.nextToken(), imports, imported)) {
		       return false;
		    }
		}
	    } else {
		if (!resolveDependancy(className, imports, imported)) {
		    return false;
		}
	    }
	    if (imports.size() > 0) {
	        Object m = imports.firstElement();
		if (m instanceof ClassManager) {
		    superClass = (ClassManager)m;
		} else {
		    superClass = ((PackageManager)m).findClass(className);
		}
	    } else {
		superClass = null;
	    }
	}
	String interfaceNames = newSpec.interfaceNames();
	if (interfaceNames != null) {
	    StringTokenizer tokenizer =
		new StringTokenizer(interfaceNames, ", \t\n\r");
	    while (tokenizer.hasMoreTokens()) {
		if (!resolveDependancy(tokenizer.nextToken(), imports, imported)) {
		   return false;
		}
	    }
	}
	header().resolveImports(imports, imported);
	return true;
    }

    /**
     * resolve class whom I depend on
     */
    private boolean resolveDependancy(String className, Vector imports, Vector imported) {
	ClassManager cm = null;
	Manager toBeImported = null;
	PackageManager pman = packageManager();
	int indexOfDot = className.lastIndexOf('.');
	if (indexOfDot != -1) {
	    // className has package part
	    String pkgName = className.substring(0, indexOfDot);
	    pman = PackageManager.findPackage(pkgName);
	    if (pman == null) {
		SystemManager.information("Package " + pkgName + " not found.\n" +
					  "Could not resolve dependancy for: " + className);
	    } else if ((cm = pman.findClass(className.substring(indexOfDot + 1))) == null) {
		SystemManager.information("Class " + className + " not found.\n" +
					  "Could not resolve dependancy for: " + className);
	    }
	} else {
	    Enumeration p = PackageManager.allPackages();
	    PackageManager pm = pman;
	    while (true) {
		cm = pm.findClass(className);
		if (cm != null) {
		    if (pm == pman || pm.name().equals("java.lang") ||
			imported.contains(cm) || imported.contains(pm)) {
		        break;
		    } else if (SystemManager.confirm(
					"Class " + className +
					" found in package " + pm.name() + ".\n",
				 	"Import", "Ignore")) {
		    /*
			if (SystemManager.confirm(
				"Import what?",
			 	pm.name() + ".*", cm.fullClassName())) {
			    toBeImported = pm;
			} else {
			    toBeImported = cm;
			}
		     */
			toBeImported = cm;
			break;
		    }
		    cm = null;
		}
		if (p.hasMoreElements()) {
		    pm = (PackageManager)p.nextElement();
		} else {
		    break;
		}
	    }
	}

	if (cm == null) {
	    if (className.indexOf('.') >= 0) {
		return true;
	    } else {
		return SystemManager.confirm(
			    "Class " + className + " has not been imported yet.\n" +
			    "Proceed it?", "Proceed", "Abort");
	    }
	} else {
	    if (!spec.isInterface()) resolveMethodsFrom(cm);
	    if (toBeImported != null) imports.addElement(toBeImported);
	    return true;
	}
    }

    /**
     * resolve methods derived my dependancy
     */
    private void resolveMethodsFrom(ClassManager depend) {
	try {
	    boolean confirmed = false;
	    boolean isInterface = depend.spec().isInterface();
	    Enumeration m = depend.methods();
	    while (m.hasMoreElements()) {
		MethodManager mm = (MethodManager)m.nextElement();
		MethodSpec mspec = mm.spec();
		if ((isInterface
		  && findMethod(mspec.identityName()) == null
		  && !hierarchy(mspec.identityName()).hasMoreElements())
		 || (!isInterface
		  && mspec.isAbstract() // there is case of interface&&abstract
		  && findMethod(mspec.identityName()) == null)) {
		    if (isInterface && !confirmed) {
			if (!SystemManager.confirm("Create methods stubs for " + depend.spec().className() + "?", "Yes", "No")) {
			    return;
			}
			confirmed = true;
		    }
		    CodeReader reader = new CodeReader(mm.getCode());
		    reader.nextDescription();
		    String desc = reader.description();
		    if (mspec.isAbstract()) {
			int head = desc.indexOf(AccessFlags.STR_ABSTRACT);
			desc = desc.substring(0, head) +
			       desc.substring(head + AccessFlags.STR_ABSTRACT.length() + 1);
		    }
		    if (isInterface && !mspec.isPublic()) {
			desc = AccessFlags.STR_PUBLIC + " " + desc;
		    }
		    MethodSpec newSpec = new MethodSpec(desc, name());
		    MethodManager newMm = new MethodManager(this, newSpec);
		    addChild(newMm);
		    StringBuffer buf = new StringBuffer();
		    buf.append("    /**\n");
		    buf.append("     * " + depend.fullClassName() + " stuff\n");
		    buf.append("     */\n");
		    buf.append("    ");
		    buf.append(desc);
		    buf.append(" {\n");
		    String value = mspec.defaultReturnValue();
		    if (value != null) {
			buf.append("\treturn ");
			buf.append(value);
			buf.append(";\n");
		    }
		    buf.append("    }\n");
		    newMm.created(buf.toString());
		}
	    }
	}
	catch (TeikadeException e) {
	}
    }

    /**
     * get class source code
     */
    protected String getCodeInternal() throws TeikadeException {
	readSource();
	if (parent == null)
	    throw new TeikadeNotFoundException("class " + name() + " seems to have been removed");
	TeikadeFile file = file();
	if (code instanceof Vector) {
	    StringBuffer buf = new StringBuffer();
	    for (Enumeration r = ((Vector)code).elements(); r.hasMoreElements(); ) {
		SourceCodeRange aRange = (SourceCodeRange)r.nextElement();
		buf.append(aRange.getCode(file));
	    }
	    String codeString = buf.toString();
	    int lineFeed = codeString.lastIndexOf(/*'{'*/'}') - 1;
	    if (lineFeed > 0) {
		char c1 = codeString.charAt(lineFeed - 1);
		char c0 = codeString.charAt(lineFeed);
		if (c1 == '\n' && c0 == '\n'
		 || c1 == '\r' && c0 == '\r'
		 || lineFeed > 3 && c1 == '\r' && c0 == '\n'
		    && codeString.charAt(lineFeed - 3) == '\r'
		    && codeString.charAt(lineFeed - 2) == '\n') {
		    codeString = codeString.substring(0, lineFeed).trim() +
				 codeString.substring(lineFeed);
		}
	    }
	    code = new StringSourceCode(codeString);
	}
	return ((SourceCodeRange)code).getCode(file);
    }

    /**
     * sync sources for the class
     */
    public void sync() {
	if (parent != null) {
	    parent.sync();
	}
    }

    /**
     * read source file
     */
    public void readSource() throws TeikadeException {
	if (parent != null) {
	    ((SourceFileManager)parent).readSource();
	}
	if (spec == null) {
	    throw new TeikadeNotFoundException("Class not found in the source file");
	}
    }

    /**
     * copy instance variables from another ClassManager
     */
    void copyFrom(ClassManager cman, boolean force, boolean verbose) throws TeikadeException {
	Vector order = new Vector();
	Hashtable hash = new Hashtable();
	Enumeration m = cman.methods();
	while (m.hasMoreElements()) {
	    MethodManager cman_mm = (MethodManager)m.nextElement();
	    MethodManager mm = findMethod(cman_mm.name());
	    if (mm != null) {
		mm.copyFrom(cman_mm, force);
		order.addElement(mm);
		hash.put(mm.name(), mm);
	    } else {
		if (verbose)
		     SystemManager.messageln("    + " + cman_mm.name());
		cman_mm.parent = this;
		order.addElement(cman_mm);
		hash.put(cman_mm.name(), cman_mm);
	    }
	}

	m = children();
	String pname = null;
	while (m.hasMoreElements()) {
	    MethodManager mm = (MethodManager)m.nextElement();
	    if (!hash.containsKey(mm.name())) {
		if (mm.isChanged()) {
		    if (pname != null) {
			MethodManager prevm = (MethodManager)hash.get(pname);
			int index = order.indexOf(prevm);
			order.insertElementAt(mm, index + 1);
		    } else {
			// not reached
			SystemManager.exception(new TeikadeException("teikade internal error"));
		    }
		    if (verbose) SystemManager.messageln("    ! " + mm.name());
		} else {
		    mm.parent = null;
		    if (verbose) SystemManager.messageln("    - " + mm.name());
		}
	    }
	    pname = mm.name();
	}

	m = order.elements();
	child = (Manager)m.nextElement();
	Manager prev = child;
	while (m.hasMoreElements()) {
	    prev.brother = (Manager)m.nextElement();
	    prev = prev.brother;
	}
	prev.brother = null;
	spec = cman.spec;
	code = cman.code;
	if (force || !(range instanceof Changes)) {
	    range = cman.range;
	}
    }

    /**
     * source file
     */
    public TeikadeFile file() {
	if (parent != null) {
	    return ((SourceFileManager)parent).file();
	} else {
	    return null;
	}
    }

    public Enumeration methods() throws TeikadeException {
	readSource();
	return children();
    }

    public Enumeration methodNames() throws TeikadeException {
	readSource();
	return childNames();
    }

    public Enumeration methodPrototypes(AccessFlags hideMethods)
    throws TeikadeException {
	readSource();
	return new MethodPrototypeEnumerator(child, hideMethods);
    }

    public int numMethods() throws TeikadeException {
	readSource();
	return numChildren();
    }

    public MethodManager findMethod(String name)
    throws TeikadeNoClassException {
	return (MethodManager)findChild(name);
    }

    public MethodManager findMethodByPrototype(String prototype)
    throws TeikadeException {
	readSource();
	Enumeration m = methods();
	while (m.hasMoreElements()) {
	    MethodManager mm = (MethodManager)m.nextElement();
	    if (mm.spec().prototype().equals(prototype)) {
		return mm;
	    }
	}
	return null;
    }

    public void changeMethodsOrder(Enumeration methods) {
	if (methods.hasMoreElements()) {
	    try {
		setCode(getCodeInternal());
		child = (Manager)methods.nextElement();
		Manager prev = child;
		while (methods.hasMoreElements()) {
		    prev.brother = (Manager)methods.nextElement();
		    prev = prev.brother;
		}
		prev.brother = null;
	    }
	    catch (TeikadeException e) {
	    }
	}
    }

    public ClassSpec spec() throws TeikadeException {
	readSource();
	return spec;
    }

    public void changed(Changes chg) {
	/*
	if (code instanceof Vector) {
	    chg.setPrevious(null);
	} else {
	    chg.setPrevious((SourceCodeRange)code);
	}
	*/
	code = chg;
	changed();
    }

    public boolean isChanged() {
	/*
	if (code == null) return false;
	if (code instanceof Vector) return false;
	 */
	return parent().isChanged();
    }

    public void changed() {
	parent.changed();
    }

    /**
     * write source code on PrintStream
     */
    public void writeOn(PrintStream ps, boolean synced) throws java.io.IOException {
	String codeString = getCode();
	int index = codeString.lastIndexOf('}');
	if (parent().child != this) ps.println("");
	ps.write(CodeConverter.toBytes(codeString.substring(0, index).trim()));
	ps.println("");
	for (Enumeration m = child.brothers(); m.hasMoreElements(); ) {
	    ((MethodManager)m.nextElement()).writeOn(ps, synced);
	}
	ps.write(CodeConverter.toBytes(codeString.substring(index).trim()));
	ps.println("");
	if (synced) {
	    ((SourceCodeRange)this.code).synced();
	    ((SourceCodeRange)this.code).dispose();
	    code = null;
	}
    }

    /**
     * return offset from beggining of this class
     */
    public int lineOffset(int line) throws TeikadeException {
	readSource();
	if (line <= range.endLine()) {
	    return line - (int)range.startLine();
	} else {
	    return line + 1;
	}
    }

    /**
     * return beggining of line's number of this class in source file
     */
    public int startLine() {
	return (int)range.startLine();
    }

    public String fullClassName() {
	PackageManager pm = packageManager();
	if (pm instanceof TopDirPackageManager) {
	    return name();
	} else {
	    return pm.name() + "." + name();
	}
    }

    public boolean hasMainMethod() {
	try {
	    if (findMethodByPrototype("void main(String[])") != null) {
		return true;
	    }
	}
	catch (TeikadeException e) {
	}
	return false;
    }

    public boolean isApplet() {
	Enumeration h = hierarchy(null);
	while (h.hasMoreElements()) {
	    CodePath cp = (CodePath)h.nextElement();
	    String fullName = cp.fullClassName();
	    if (fullName.startsWith("java.")) {
		return fullName.equals("java.applet.Applet");
	    }
	}
	return false;
    }

    public boolean isInterface() {
	try {
	    if (spec().isInterface()) {
		return true;
	    }
	}
	catch (TeikadeException e) {
	}
	return false;
    }

    /**
     * execute the class
     */
    public boolean execute(String args, teikade.debug.DebuggerManager debugger) {
	if (!canExecute) return false;
	if (args == null) args = "";
	TeikadeFile classFile = binaryFile(((SourceFileManager)parent).directory());
	if (!classFile.exists()) {
	    SystemManager.information("no class file\nneed to be compiled before execute");
	    return false;
	}
	TeikadeFile sourceFile = ((SourceFileManager)parent).file();
	if (sourceFile.lastModified() > classFile.lastModified()) {
	    if (SystemManager.confirm("source file is newer than compiled class file\n" +
				      "should compile before execute",
				      "Abort to execute", "Execute")) {
		return false;
	    }
	}
	String name = (header().packageName() == null) ? this.name : fullClassName();
	String classpath = classPathProperty();
	if (classpath == null) {
	    classpath = projectManager().path;
	}
	classpath += SystemManager.PATHSEPARATOR + SystemManager.CLASSPATH;
	try {
	    if (hasMainMethod()) {
		// execute main method
		if (java == null) {
		    SystemManager.information("No interpritor found. Abort.");
		    return false;
		}
		if (debugger == null) {
		    String cmd = java;
		    if (SystemManager.onOS2
			&& (isApplet() || SystemManager.confirm("Does this class use AWT?\n", "Use AWT", "No"))) {
			cmd = javapm;
		    }
		    cmd += " -classpath " + classpath + " " + name + " " + args;
		    if ("true".equals(System.getProperty("debug.execute"))) {
			System.err.println("debug.execute: " + cmd);
		    }
		    new Terminal(cmd, "Execution: " + name).show();
		} else {
		    debugger.runClass(name, classpath, args);
		}
		return true;
	    }
	    // execute applet
	    ClassSpec spec;
	    try {
		spec = spec();
	    }
	    catch (TeikadeException e) {
		spec = null;
	    }
	    if (spec == null) {
		SystemManager.information("Class " + name + " has no spec", "Abort");
		return false;
	    }
	    String superClassName = spec.superClassName();
	    if (isApplet()) {
		File tmpdir = new File(SystemManager.profileDirectory(), ".tmp");
		if (!tmpdir.exists()) {
		    if (tmpdir.mkdir()) {
			SystemManager.information("Created temporaly directory for applets in profile directory.");
		    } else {
			SystemManager.information("Could not create temporaly directory for applets in profile directory.\nAborted");
			return false;
		    }
		}
		String fname = tmpdir.getPath() + File.separator +
			       name + "." + idNum++ + ".html";
		PrintStream html = new PrintStream(new BufferedOutputStream(new FileOutputStream(fname)));
		try {
		    html.print("<APPLET CODE=\"");
		    html.write(CodeConverter.toBytes(name));
		    html.print("\" ");
		    if (args.indexOf("width=")  < 0) html.print("width=100 ");
		    if (args.indexOf("height=") < 0) html.print("height=100 ");
		    html.write(CodeConverter.toBytes(args));
		    html.print(">\n</APPLET>\n");
		} finally {
		    html.close();
		}
		if (debugger == null) {
		    String url;
		    if (SystemManager.onWin32) {
			classpath = "\"" + classpath + "\"";
			url = "\"" + "file:\\" + fname + "\"";
		    } else {
			url = fname.replace(File.separatorChar, '/');
			url = url.replace(':', '|');
			url = "file://" + url;
			if (SystemManager.onOS2) {
			    classpath = "\"" + classpath + "\"";
			    url = "\"" + url + "\"";
			}		    
		    }
		    String cmd = java;
		    if (SystemManager.onOS2) cmd = javapm;
		    cmd += " -classpath " + classpath +
			   " sun.applet.AppletViewer " + url;
		    if ("true".equals(System.getProperty("debug.execute"))) {
			System.err.println("debug.execute: " + cmd);
		    }
		    new Terminal(cmd, "Execution: " + name).show();
		} else {
		    debugger.runApplet(name, classpath, fname);
		}
		return true;
	    }
	} catch (IOException e) {
	    SystemManager.information("Failed to execute " + name, "Abort");
	}
	return false;
    }

    /**
     * execute the class
     */
    public void executeWithHTML(String html, teikade.debug.DebuggerManager debugger) {
	if (html == null) html = "";
	String name = fullClassName();
	TeikadeFile f = TeikadeFile.create(((SourceFileManager)parent).absolutePath());
	String classpath = f.getParent();
	TeikadeFile classFile = TeikadeFile.create(classpath + File.separator + name() + ".class");
	if (!classFile.exists()) {
	    SystemManager.information("no class file\nneed to compile before execute");
	    return;
	}
	if (f.lastModified() > classFile.lastModified()) {
	    if (SystemManager.confirm("source file is newer than compiled class file\n" +
				      "should compile before execute",
				      "Abort to execute", "Execute")) {
		return;
	    }
	}
	if (header().packageName() != null) {
	    classpath = projectManager().absolutePath();
	} else {
	    name = this.name;
	}
	classpath = classpath + SystemManager.PATHSEPARATOR +
				SystemManager.CLASSPATH;
	ClassSpec spec;
	try {
	    spec = spec();
	}
	catch (TeikadeException e) {
	    spec = null;
	}
	if (spec == null) {
	    SystemManager.information("Class " + name + " has no spec", "Abort");
	    return;
	}
	String superClassName = spec.superClassName();
	if (isApplet()) {
	    if (debugger == null) {
		String url;
		if (SystemManager.onWin32) {
		    url = "\"" + "file:\\" + html + "\"";
		} else {
		    url = html.replace(File.separatorChar, '/');
		    url = url.replace(':', '|');
		    url = "file://" + url;
		    if (SystemManager.onOS2) {
			url = "\"" + url + "\"";
		    }		    
		}
		String cmd = java;
		if (SystemManager.onOS2) cmd = javapm;
		cmd += " -classpath " + classpath +
		       " sun.applet.AppletViewer " + url;
		try {
		    if ("true".equals(System.getProperty("debug.execute"))) {
			System.err.println("debug.execute: " + cmd);
		    }
		    new Terminal(cmd, "Execution: " + name).show();
		} catch (IOException e) {
		    SystemManager.information("Failed to execute " + name, "Abort");
		}
	    } else {
		debugger.runApplet(name, classpath, html);
	    }
	} else {
	    SystemManager.information(name + " is not an applet.", "Abort");
	}
    }

    /**
     * rename the class
     */
    public Manager rename(String newName)
    throws TeikadeException {
	if (packageManager().findClass(newName) != null) {
	    SystemManager.information("Class " + newName + " already exists. Aborted.");
	    return null;
	}
	removeClassFiles();
	if (spec.isPublic()) try {
	    parent.rename(newName + ".java");
	}
	catch (TeikadeException e) {
	    SystemManager.information(e.getMessage() + ". Aborted");
	    return null;
	}
	Enumeration m = methods();
	while (m.hasMoreElements()) {
	    MethodManager mm = (MethodManager)m.nextElement();
	    if (mm.spec().isConstructor()) {
		mm.rename(newName);
	    }
	}
	String code = getCode();
	StringBuffer newCode = new StringBuffer();
	CodeReader reader = new CodeReader(code, 1, false);
	reader.nextDescription();
	int index = code.indexOf(ClassSpec.STR_CLASS, reader.startPoint());
	if (index != -1) {
	    index += ClassSpec.STR_CLASS.length();
	} else {
	    index = code.indexOf(ClassSpec.STR_INTERFACE, reader.startPoint()) +
			ClassSpec.STR_INTERFACE.length();
	}
	index = code.indexOf(name, index);
	newCode.append(code.substring(0, index));
	newCode.append(newName);
	newCode.append(code.substring(index + name.length()));
	spec.rename(newName);
	name = newName;
	return renamed(newCode.toString());
    }

    /**
     * rename the class
     */
    public Manager removing() {
	TeikadeFile dir = binaryPath();
	String name = this.name;
	Manager manager = super.removing();
	if (manager == null) {	// manager was removed
	    removeClassFiles(dir, name);
	}
	return manager;
    }

    /**
     * remove class files
     */
    public void removeClassFiles() {
	removeClassFiles(binaryPath(), name);
    }

    /**
     * remove class files
     */
    public void removeClassFiles(TeikadeFile dir, String name) {
	TeikadeFile.create(dir, name + ".class").delete();
	String innerName = name + "$";
	String[] list = dir.list();
	for (int i = 0; i < list.length; i++) {
	    if (list[i].startsWith(innerName)) {
		TeikadeFile.create(dir, list[i]).delete();
	    }
	}
    }

    /**
     * dispose all information
     */
    protected void dispose() {
	//disposeDetails();
	super.dispose();
    }

    /**
     * dispose class specific data
     */
    protected void disposeDetails() {
	if (spec != null) {
	    spec.dispose();
	    spec = null;
	}
	if (code instanceof Vector) {
	    Enumeration e = ((Vector)code).elements();
	    while (e.hasMoreElements()) {
		SourceCodeRange c = (SourceCodeRange)e.nextElement();
		c.dispose();
	    }
	    code = null;
	} else if (code instanceof SourceCodeRange) {
	    ((SourceCodeRange)code).dispose();
	    code = null;
	}
	if (range != null) {
	    range.dispose();
	    range = null;
	}
	if (child != null && child.brother != null) {
	    ((MethodManager)child.brother).disposeDetails();
	}
	if (brother != null) {
	    ((ClassManager)brother).disposeDetails();
	}
    }
}
