/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.sys;

import java.io.*;

public class TeikadeFile extends File {
    private static final boolean useRename = SystemManager.onSolaris;

    public static TeikadeFile create(String path) {
	return new TeikadeFile(path);
    }

    public static TeikadeFile create(String path, String name) {
	return create(path + File.separator + name);
    }

    public static TeikadeFile create(File file, String name) {
	String path = file.getAbsolutePath();
	if (path.endsWith(File.separator)) {
	    return create(path + name);
	} else {
	    return create(path + File.separator + name);
	}
    }

    protected TeikadeFile(String path) {
        super(path);
    }

    protected TeikadeFile(String path, String name) {
        super(path, name);
    }

    protected TeikadeFile(File dir, String name) {
        super(dir, name);
    }

    /*
     * for Win32 platforms:
     * They didn't override existing file, so we must copy all contents ;-<
     */
    public boolean renameTo(File dest){
	if (!exists()) return false;
	if (dest.exists()) dest.delete();
	return super.renameTo(dest);
    }
 

    /*
     * for OS/2 platform:
     * File#canRead() and File#canWrite() for directory always return 'false' (;_;)
     */
    public boolean canRead() {
	if ( SystemManager.onOS2 && isDirectory() ) {
	    String javaver = System.getProperty( "java.version" );
	    if ( 19960520 >= Integer.parseInt( javaver.substring( javaver.length()-8 ) ) ) {
		return true;
	    } else {
		return super.canRead();
	    }
	} else {
	    return super.canRead();
	}
    }

    public boolean canWrite() {
	if ( SystemManager.onOS2 && isDirectory() ) {
	    String javaver = System.getProperty( "java.version" );
	    if ( 19960520 >= Integer.parseInt( javaver.substring( javaver.length()-8 ) ) ) {
		return true;
	    } else {
		return super.canWrite();
	    }
	} else {
	    return super.canWrite();
	}
    }

/*
    public Writer getWriter() throws IOException {
	return new FileWriter(this);
    }

    public Reader getReader() throws IOException {
	return new FileReader(this);
    }
*/

    public OutputStream getOutputStream() throws IOException {
	return new FileOutputStream(this);
    }

    public InputStream getInputStream() throws IOException {
	return new FileInputStream(this);
    }
}
