/*
 * Copyright (c) 1995 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.*;
import teikade.lang.*;
import teikade.gui.TeikadeFrame;
import teikade.gui.editor.EditProperties;
import java.lang.*;
import java.util.*;
import java.io.*;

public class CodeLocation implements ListItem {

    CodePath codePath;
    private String listString;
    private int lineNumber;
    private int cursorOffset;
    private int cursorLength;

    boolean isMarked = false;

    public CodeLocation(String fname, int line, String item)
						throws CodeLocationException {
        this(fname, line, item, 0, -1);
    }

    public CodeLocation(String fname, int line, String item,
		        int offset, int length) throws CodeLocationException {
	listString  = item;
	cursorOffset = offset;
	cursorLength = length;

	SourceFileManager sourceFileManager =
			SourceFileManager.findSourceFileManagerForFile(fname);
	try {
	    codePath = searchCodePath(sourceFileManager, line);
	}
	catch (TeikadeException e) {
	    codePath = null;
	}
	if (codePath == null) throw new CodeLocationException();
    }

    public CodeLocation(CodePath cp, String fname, int line, String item,
			int offset, int length) throws CodeLocationException {
	listString = item;
	cursorOffset = offset;
	cursorLength = length;

	PackageManager packageManager = cp.packageManager();
	if (packageManager != null) {
	    SourceFileManager sourceFileManager = packageManager.findSource(fname);
	    try {
		codePath = searchCodePath(sourceFileManager, line);
		if (codePath != null) return;
	    }
	    catch (TeikadeException e) {
	    }
	}
	    
	ProjectManager projectManager = cp.projectManager();
	if (projectManager != null) {
	    Enumeration packages = projectManager.packages();
	    while (packages.hasMoreElements()) {
		SourceFileManager sourceFileManager =
		    ((PackageManager)packages.nextElement()).findSource(fname);
		try {
		    codePath = searchCodePath(sourceFileManager, line);
		    if (codePath != null) return;
		}
		catch (TeikadeException e) {
		}
	    }
	}

	SourceFileManager sourceFileManager =
			SourceFileManager.findSourceFileManagerForFile(fname);
	try {
	    codePath = searchCodePath(sourceFileManager, line);
	}
	catch (TeikadeException e) {
	    codePath = null;
	}
	if (codePath == null) throw new CodeLocationException();
    }

    public CodeLocation(CodeLocation clone) {
	codePath     = new CodePath(clone.codePath);
	listString   = clone.listString;
	lineNumber   = clone.lineNumber;
	cursorOffset = clone.cursorOffset;
	cursorLength = clone.cursorLength;
	isMarked     = clone.isMarked;
    }

    public ListItem cloneItem() {
	return new CodeLocation(this);
    }

    private CodePath searchCodePath(SourceFileManager sourceFileManager, int line) throws TeikadeException {
	if (sourceFileManager == null) return null;
	lineNumber = 0;
	Enumeration classes = sourceFileManager.classes();
	while (classes.hasMoreElements()) {
	    ClassManager classManager = (ClassManager)classes.nextElement();
	    if ((lineNumber = classManager.lineOffset(line)) <= line) {
		    Enumeration methods = classManager.methods();
		    while (methods.hasMoreElements()) {
			MethodManager methodManager = (MethodManager)methods.nextElement();
			int lineOffset;
			if ((lineOffset = methodManager.lineOffset(line)) >= 0) {
			    lineNumber = lineOffset;
			    return new CodePath(methodManager);
			}
		    }
		return new CodePath(classManager);
	    }
	}
	return new CodePath(sourceFileManager);
    }

    public CodePath codePath() {
	return codePath;
    }

    public String listString() {
	return listString;
    }

    private static final int nextLine(String str, int pos, int end) {
	char c;
	while (pos < end) {
	    c = str.charAt(pos++);
	    if (c == '\n') {
		return pos;
	    } else if (c == '\r') {
		if (pos < end && str.charAt(pos) == '\n') {
		    pos++;
		}
		return pos;
	    }
	}
	return -1;
    }

    public int cursorPos()[] {
	int cursor[] = new int[2];
	cursor[0] = 0;
	cursor[1] = 0;
	if (codePath == null) return cursor;
	String code = codePath.getManager().getCode();
	int length = code.length();
	char c;
	int bol = 0;
	for (int line = lineNumber; line > 0; line--) {
	    if ((bol = nextLine(code, bol, length)) < 0) {
		cursor[0] = cursor[1] = length;
		return cursor;
	    }
	}
	int fileTab = EditProperties.getFileTabWidth(codePath.projectName());
	if (fileTab == 8) {
	    cursor[0] = bol + cursorOffset;
	    if (cursorLength >= 0) {
		cursor[1] = cursor[0] + cursorLength;
	    } else {
		cursor[1] = nextLine(code, bol, length);
		if (cursor[1] < 0) cursor[1] = length;
	    }
	} else {
	    int eol = nextLine(code, bol, length);
	    if (eol < 0) eol = length;
	    String line = code.substring(bol, eol);
	    bol = TabConverter.convert(code.substring(0, bol), fileTab, 8).length();
	    cursor[0] = bol;
	    if (cursorOffset > 0) {
		cursor[0] += TabConverter.index(line, cursorOffset, 8, fileTab);
	    }
	    if (cursorLength == 0) {
		cursor[1] = cursor[0];
	    } else if (cursorLength > 0) {
		cursor[1] = bol + TabConverter.index(line, cursorOffset + cursorLength, 8, fileTab);
	    } else {
		cursor[1] = bol + TabConverter.convert(line, fileTab, 8).length();
	    }
	}
	return cursor;
    }

    public boolean isMarked() {
	return isMarked;
    }

    public boolean isChanged() {
	return true;
    }

    public boolean isSameCode(ListItem item) {
	return codePath().matches(item.codePath());
    }

    public void setMark(boolean marked) {
	isMarked = marked;
    }
}
