/*
 * Copyright (c) 1998 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.SystemManager;
import teikade.sys.CodePath;
import teikade.gui.editor.EditProperties;

public class CompileJDKResultBuilder implements CommandExternalListener {
    private int sequence;
    final StringBuffer messages = new StringBuffer();
    int type;
    String fileName;
    String errorText;
    int lineNumber;
    int offsetNumber;
    String description;
    Compile client;

    public CompileJDKResultBuilder(Compile client) {
	this.client = client;
	reset();
    }

    protected void reset() {
	sequence = 0;
	messages.setLength(0);
	type = CompileResult.MESSAGE;
	fileName = null;
	lineNumber = 0;
	offsetNumber = 0;
	description = null;
    }

    public void output(Command command, String message) {
	switch (++sequence) {
	case 1:
	    if (!setFileLineReason(message)) {
		postMessage(message);
	    }
	    break;
	case 2:
	    if (!setSource(message)) {
		postMessage(message);
	    }
	    break;
	case 3:
	    setOffset(message);
	    postResult();
	    break;
	}
    }

    public void error(Command command, String message) {
	output(command, message);
    }


    protected void addMessage(String message) {
	messages.append(message);
    }

    protected String getMessage() {
	return messages.toString();
    }

    protected boolean setFileLineReason(String message) {
	int first  = message.indexOf(':');
	if (first < 0) return false;
	if ((SystemManager.onWin32 || SystemManager.onOS2) &&
	    first == 1 && message.charAt(2) == '\\') {
	    char c = message.charAt(0);
	    if ('A' <= c && c <= 'Z' || 'a' <= c && c <= 'z') {
		first = message.indexOf(':', 3);
	    }
	}
	int second = message.indexOf(':', first + 1);
	if (second <= first) return false;
	fileName = message.substring(0, first);
	try {
	    lineNumber = Integer.parseInt(message.substring(first + 1, second));
	} catch (NumberFormatException e) {
	    return false;
	}
	description = message.substring(second + 1).trim();
	if (description.startsWith("Note:")) {
	    description = description.substring(5).trim();
	    type = CompileResult.WARNING;
	    addMessage("warning: ");
	} else {
	    type = CompileResult.ERROR;
	    addMessage("error: ");
	}
	addMessage(description);
	addMessage("\n");
	return true;
    }

    protected boolean setSource(String message) {
	errorText = message;
	addMessage(message);
	addMessage("\n");
	return true;
    }

    protected boolean setOffset(String message) {
	addMessage(message);
	addMessage("\n");
/*
	int fileTab = 8;
	try {
	    CodeLocation location = new CodeLocation(fileName, lineNumber, "");
	    CodePath codePath = location.codePath();
	    String projectName = codePath.projectName();
	    if (projectName != null) {
		fileTab = EditProperties.getFileTabWidth(projectName);
	    }
	} catch (CodeLocationException e) {
	}
*/
	offsetNumber = message.indexOf('^');
	if (offsetNumber <= 0) {
	    offsetNumber = 0;
	}
/*
 else if (fileTab != 8) {
	    offsetNumber = TabConverter.index(errorText, offsetNumber, 8, fileTab);
	}
*/
	return true;
    }

    public void flush() {
	postMessage();
    }

    protected void postMessage(String message) {
	addMessage(message);
	addMessage("\n");
	postMessage();
    }

    public void postMessage() {
	String message = getMessage();
	if (message.length() > 0) {
	    client.result(new CompileResult(message));
	}
	reset();
    }

    public void postResult() {
	client.result(new CompileResult(type, fileName, lineNumber,
				offsetNumber, description, getMessage()));
	reset();
    }

    public void executing(Command command) {
	client.executing();
    }

    public void finished(Command command, int value) {
	flush();
	client.finished(value);
    }

    public void aborted(Command command, Exception e) {
	flush();
	client.aborted(e);
    }
}
