/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.util;

import java.util.Properties;
import java.util.Observable;
import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.Font;
import java.awt.Color;
import java.io.*;

/*
 * system properties for teikade
 */
public class ParsedProperties extends Properties {

    private static Hashtable _colors;

    private static Hashtable getColors() {
	if (_colors == null) {
	    _colors = new Hashtable();
	    _colors.put("white",	Color.white);
	    _colors.put("lightgray",	Color.lightGray);
	    _colors.put("gray",		Color.gray);
	    _colors.put("darkgray",	Color.darkGray);
	    _colors.put("black",	Color.black);
	    _colors.put("red",		Color.red);
	    _colors.put("pink",		Color.pink);
	    _colors.put("orange",	Color.orange);
	    _colors.put("yellow",	Color.yellow);
	    _colors.put("green",	Color.green);
	    _colors.put("magenta",	Color.magenta);
	    _colors.put("cyan",		Color.cyan);
	    _colors.put("blue",		Color.blue);
	}
	return _colors;
    }

    public ParsedProperties() {
	super();
    }

    public ParsedProperties(Properties defaults) {
	super(defaults);
    }

    /**
     * java.util.Properties has a stupid mistake. So I have to writeit all...
     * Loads properties from an InputStream.
     * @param in the input stream
     * @exception IOException Error when reading from input stream.
     */
    public synchronized void load(InputStream in) throws IOException {
	in = Runtime.getRuntime().getLocalizedInputStream(in);

	int ch = in.read();
	while (true) {
	    switch (ch) {
	      case -1:
		return;

	      case '#':
	      case '!':
		do {
		    ch = in.read();
		} while ((ch >= 0) && (ch != '\n') && (ch != '\r'));
		continue;

	      case '\n':
	      case '\r':
	      case ' ':
	      case '\t':
		ch = in.read();
		continue;
	    }

	    // Read the key
	    StringBuffer key = new StringBuffer();
	    while ((ch >= 0) && (ch != '=') && (ch != ':') && 
		   (ch != ' ') && (ch != '\t') && (ch != '\n') && (ch != '\r')) {
		key.append((char)ch);
		ch = in.read();
	    }
	    while ((ch == ' ') || (ch == '\t')) {
		ch = in.read();
	    }
	    if ((ch == '=') || (ch == ':')) {
		ch = in.read();
	    }
	    while ((ch == ' ') || (ch == '\t')) {
		ch = in.read();
	    }

	    // Read the value
	    StringBuffer val = new StringBuffer();
	    while ((ch >= 0) && (ch != '\n') && (ch != '\r')) {
		if (ch == '\\') {
		    switch (ch = in.read()) {
		      case '\n': 
			while (((ch = in.read()) == ' ') || (ch == '\t'));
			continue;
		      case 't': ch = '\t'; break;
		      case 'n': ch = '\n'; break;
		      case 'r': ch = '\r'; break;
		      case 'u': {
			while ((ch = in.read()) == 'u');
			int d = 0;
		      loop:
			for (int i = 0 ; i < 4 ; i++, ch = in.read()) {
			    switch (ch) {
			      case '0': case '1': case '2': case '3': case '4':
			      case '5': case '6': case '7': case '8': case '9':
				d = (d << 4) + ch - '0';
				break;
			      case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
				d = (d << 4) + 10 + ch - 'a';
				break;
			      case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
				d = (d << 4) + 10 + ch - 'A';
				break;
			      default:
				break loop;
			    }	
			}
			ch = d;
		      }
		    }
		}
		val.append((char)ch);
		ch = in.read();
	    }

	    put(key.toString(), val.toString());
	}
    }

    /**
     * Get String property
     */
    public String getParsedString(String key, String defaultKey) {
	String name = getString(key, defaultKey);
	if (name != null) {
	    StringBuffer buf = new StringBuffer();
	    int from = 0;
	    int last = 0;
	    while ((from = name.indexOf('%', last)) > 0) {
		if (last < from) {
		    buf.append(name.substring(last, from));
	        }
		from++;
		last = name.indexOf('%', from);
		if (last < from) {
		    System.out.println("Warning: property syntax error: " + name);
		    last = from;
		    break;
		}
		String propKey = name.substring(from, last);
		String value = getString(propKey);
		if (value != null) {
		    buf.append(value);
		} else {
		    // we should do something, but what?
		}
		last++;
	    }
	    buf.append(name.substring(last));
	    return buf.toString().replace('/', File.separatorChar);
	} else {
	    return name;
	}
    }

    /**
     * Get String property
     */
    public String getParsedString(String key) {
	return getParsedString(key, null);
    }

    /**
     * Get String property
     */
    public String getString(String key, String defaultKey) {
	String name = getProperty(key);
	if (name == null && defaultKey != null) {
	    name = getProperty(defaultKey);
	}
	return name;
    }

    /**
     * Get String property
     */
    public String getString(String key) {
	return getString(key, null);
    }

    /**
     * Get integer property
     */
    public int getInteger(String key, String defaultKey) {
	try {
	    return Integer.parseInt(getString(key));	
	} catch (Exception e0) {
	    try {
		return Integer.parseInt(getString(defaultKey));
	    } catch (Exception e1) {
		return 0;
	    }
	}
    }

    /**
     * Get integer property
     */
    public int getInteger(String key) {
	try {
	    return Integer.parseInt(getString(key));	
	} catch (Exception e) {
	    return 0;
	}
    }

    /**
     * Get font property and return proper Font.
     */
    public Font getFont(String key, int face) {
	String name = getString(key + ".name", "font.default.name");
	int size = getInteger(key + ".size", "font.default.size");
	
	return new Font(name, face, size);
    }

    /**
     * Get font property and return proper Font.
     */
    public Font getFont(String key) {
	String name = getString(key + ".name", "font.default.name");
	int face = getInteger(key + ".face", "font.default.face");
	int size = getInteger(key + ".size", "font.default.size");
	
	return new Font(name, face, size);
    }

    public Color getColor(String key) {
	String name = getString(key);
	return name == null ? null : (Color)getColors().get(name.toLowerCase());
    }

    public Enumeration getColorNames() {
	return getColors().keys();
    }
}
